# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import inspect
import logging

from tempest.common import credentials_factory as common_creds

from magnum.tests.functional.common import base
from magnum.tests.functional.common import config
from magnum.tests.functional.common import manager


COPY_LOG_HELPER = "magnum/tests/contrib/copy_instance_logs.sh"


class BaseTempestTest(base.BaseMagnumTest):
    """Sets up configuration required for functional tests"""

    ic_class_list = []
    ic_method_list = []
    LOG = logging.getLogger(__name__)

    def __init__(self, *args, **kwargs):
        super(BaseTempestTest, self).__init__(*args, **kwargs)

    @classmethod
    def setUpClass(cls):
        super(BaseTempestTest, cls).setUpClass()
        config.Config.setUp()

    @classmethod
    def tearDownClass(cls):
        super(BaseTempestTest, cls).tearDownClass()
        cls.clear_credentials(clear_class_creds=True)

    def tearDown(self):
        super(BaseTempestTest, self).tearDown()
        self.clear_credentials(clear_method_creds=True)

    @classmethod
    def clear_credentials(cls,
                          clear_class_creds=False,
                          clear_method_creds=False):
        if clear_class_creds:
            for ic in cls.ic_class_list:
                ic.clear_creds()
        if clear_method_creds:
            for ic in cls.ic_method_list:
                ic.clear_creds()

    @classmethod
    def get_credentials(cls, name=None,
                        type_of_creds="default",
                        class_cleanup=False):
        (creds, _) = cls.get_credentials_with_keypair(name, type_of_creds,
                                                      class_cleanup)
        return creds

    @classmethod
    def get_credentials_with_keypair(cls, name=None,
                                     type_of_creds="default",
                                     class_cleanup=False):
        if name is None:
            # Get name of test method
            name = inspect.stack()[1][3]
            if len(name) > 32:
                name = name[0:32]

        # Choose type of isolated creds
        ic = common_creds.get_credentials_provider(
            name,
            identity_version=config.Config.auth_version
        )

        if class_cleanup:
            cls.ic_class_list.append(ic)
        else:
            cls.ic_method_list.append(ic)

        creds = None
        if "admin" == type_of_creds:
            creds = ic.get_admin_creds()
        elif "alt" == type_of_creds:
            creds = ic.get_alt_creds()
        elif "default" == type_of_creds:
            creds = ic.get_primary_creds()
        else:
            creds = ic.self.get_credentials(type_of_creds)

        _, keypairs_client = cls.get_clients(
            creds, type_of_creds, 'keypair_setup')

        keypair = None
        try:
            keypairs_client.show_keypair(config.Config.keypair_id)
        except Exception:
            keypair_body = keypairs_client.create_keypair(
                name=config.Config.keypair_id)
            cls.LOG.debug("Keypair body: %s", keypair_body)
            keypair = keypair_body['keypair']['private_key']
        return (creds, keypair)

    @classmethod
    def get_clients(cls, creds, type_of_creds, request_type):
        if "admin" == type_of_creds:
            manager_inst = manager.AdminManager(credentials=creds.credentials,
                                                request_type=request_type)
        elif "alt" == type_of_creds:
            manager_inst = manager.AltManager(credentials=creds.credentials,
                                              request_type=request_type)
        elif "default" == type_of_creds:
            manager_inst = manager.DefaultManager(
                credentials=creds.credentials, request_type=request_type)
        else:
            manager_inst = manager.DefaultManager(
                credentials=creds.credentials, request_type=request_type)

        # create client with isolated creds
        return (manager_inst.client, manager_inst.keypairs_client)

    @classmethod
    def get_clients_with_existing_creds(cls,
                                        name=None,
                                        creds=None,
                                        type_of_creds="default",
                                        request_type=None,
                                        class_cleanup=False):
        if creds is None:
            return cls.get_clients_with_new_creds(name,
                                                  type_of_creds,
                                                  request_type,
                                                  class_cleanup)
        else:
            return cls.get_clients(creds, type_of_creds, request_type)

    @classmethod
    def get_clients_with_new_creds(cls,
                                   name=None,
                                   type_of_creds="default",
                                   request_type=None,
                                   class_cleanup=False):
        """Creates isolated creds.

        :param name: name, will be used for dynamic creds
        :param type_of_creds: admin, alt or default
        :param request_type: ClusterTemplate or service
        :returns: MagnumClient -- client with isolated creds.
        :returns: KeypairClient -- allows for creating of keypairs
        """
        creds = cls.get_credentials(name, type_of_creds, class_cleanup)
        return cls.get_clients(creds, type_of_creds, request_type)



# Copyright 2020 Catalyst IT LTD.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

"""separated CA cert for etcd and front-proxy

Revision ID: 7da8489d6a68
Revises: f1d8b0ab8b8d
Create Date: 2020-08-19 17:18:27.634467

"""

# revision identifiers, used by Alembic.
revision = '7da8489d6a68'
down_revision = 'f1d8b0ab8b8d'

from alembic import op  # noqa: E402  # noqa: E402

from sqlalchemy.types import String  # noqa: E402

import sqlalchemy as sa  # noqa: E402


def upgrade():
    op.add_column('cluster', sa.Column('etcd_ca_cert_ref',
                  String(512),
                  nullable=True))
    op.add_column('cluster', sa.Column('front_proxy_ca_cert_ref',
                  String(512),
                  nullable=True))



# Copyright 2018 Catalyst Cloud Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock

import heatclient.exc as heat_exc

from magnum.common import exception
from magnum.common import octavia
from magnum import objects
from magnum.tests import base
from magnum.tests.unit.db import utils


class TestHeatLBResource(object):
    def __init__(self, physical_resource_id):
        self.physical_resource_id = physical_resource_id


class OctaviaTest(base.TestCase):
    def setUp(self):
        super(OctaviaTest, self).setUp()

        cluster_dict = utils.get_test_cluster(node_count=1)
        nodegroups_dict = utils.get_nodegroups_for_cluster(node_count=1)
        self.cluster = objects.Cluster(self.context, **cluster_dict)
        self.nodegroups = [
            objects.NodeGroup(self.context, **nodegroups_dict['master']),
            objects.NodeGroup(self.context, **nodegroups_dict['worker'])
        ]

    @mock.patch("magnum.common.neutron.delete_floatingip")
    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_loadbalancers(self, mock_clients, mock_delete_fip):
        mock_lbs = {
            "loadbalancers": [
                {
                    "id": "fake_id_1",
                    "description": "Kubernetes external service "
                                   "ad3080723f1c211e88adbfa163ee1203 from "
                                   "cluster %s" % self.cluster.uuid,
                    "name": "fake_name_1",
                    "provisioning_status": "ACTIVE",
                    "vip_port_id": "b4ca07d1-a31e-43e2-891a-7d14f419f342"
                },
                {
                    "id": "fake_id_2",
                    "description": "Kubernetes Ingress test-octavia-ingress "
                                   "in namespace default from cluster %s, "
                                   "version: 32207" % self.cluster.uuid,
                    "name": "fake_name_2",
                    "provisioning_status": "ERROR",
                    "vip_port_id": "c17c1a6e-1868-11e9-84cd-00224d6b7bc1"
                },
            ]
        }
        mock_octavia_client = mock.MagicMock()
        mock_octavia_client.load_balancer_list.side_effect = [
            mock_lbs, {"loadbalancers": []}
        ]
        mock_octavia_client.load_balancer_show.return_value = {
            'id': 'heat_lb_id',
            'provisioning_status': 'ACTIVE'
        }

        mock_heat_client = mock.MagicMock()
        mock_heat_client.resources.list.return_value = [
            TestHeatLBResource('heat_lb_id')
        ]

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.octavia.return_value = mock_octavia_client
        osc.heat.return_value = mock_heat_client

        octavia.delete_loadbalancers(self.context, self.cluster)

        calls = [
            mock.call("fake_id_1", cascade=True),
            mock.call("fake_id_2", cascade=True),
            mock.call("heat_lb_id", cascade=True)
        ]
        mock_octavia_client.load_balancer_delete.assert_has_calls(calls)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_loadbalancers_no_candidate(self, mock_clients):
        mock_lbs = {
            "loadbalancers": []
        }
        mock_octavia_client = mock.MagicMock()
        mock_octavia_client.load_balancer_list.return_value = mock_lbs

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.octavia.return_value = mock_octavia_client

        octavia.delete_loadbalancers(self.context, self.cluster)

        self.assertFalse(mock_octavia_client.load_balancer_delete.called)

    @mock.patch("magnum.common.neutron.delete_floatingip")
    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_loadbalancers_timeout(self, mock_clients, mock_delete_fip):
        # don't wait the full 60 seconds for a unit test
        self.config(pre_delete_lb_timeout=1, group="cluster")
        osc = mock.MagicMock()
        mock_clients.return_value = osc
        mock_octavia_client = mock.MagicMock()
        osc.octavia.return_value = mock_octavia_client

        mock_lbs = {
            "loadbalancers": [
                {
                    "id": "fake_id_1",
                    "description": "Kubernetes external service "
                                   "ad3080723f1c211e88adbfa163ee1203 from "
                                   "cluster %s" % self.cluster.uuid,
                    "name": "fake_name_1",
                    "provisioning_status": "ACTIVE",
                    "vip_port_id": "b4ca07d1-a31e-43e2-891a-7d14f419f342"
                },
                {
                    "id": "fake_id_2",
                    "description": "Kubernetes external service "
                                   "a9f9ba08cf28811e89547fa163ea824f from "
                                   "cluster %s" % self.cluster.uuid,
                    "name": "fake_name_2",
                    "provisioning_status": "ACTIVE",
                    "vip_port_id": "c17c1a6e-1868-11e9-84cd-00224d6b7bc1"
                },
            ]
        }
        mock_octavia_client.load_balancer_list.return_value = mock_lbs

        self.assertRaises(
            exception.PreDeletionFailed,
            octavia.delete_loadbalancers,
            self.context,
            self.cluster
        )

    @mock.patch("magnum.common.neutron.delete_floatingip")
    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_loadbalancers_already_deleted(self, mock_clients,
                                                  mock_delete_fip):
        mock_octavia_client = mock.MagicMock()
        mock_octavia_client.load_balancer_list.return_value = {
            "loadbalancers": []
        }

        mock_heat_client = mock.MagicMock()
        mock_heat_client.resources.list.return_value = [
            TestHeatLBResource(None)
        ]

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.octavia.return_value = mock_octavia_client
        osc.heat.return_value = mock_heat_client

        octavia.delete_loadbalancers(self.context, self.cluster)

        self.assertFalse(mock_octavia_client.load_balancer_show.called)
        self.assertFalse(mock_octavia_client.load_balancer_delete.called)

    @mock.patch("magnum.common.neutron.delete_floatingip")
    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_loadbalancers_with_stack_not_found(self, mock_clients,
                                                       mock_delete_fip):
        mock_octavia_client = mock.MagicMock()
        mock_octavia_client.load_balancer_list.return_value = {
            "loadbalancers": []
        }

        mock_heat_client = mock.MagicMock()
        mock_heat_client.resources.list.side_effect = \
            heat_exc.HTTPNotFound

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.octavia.return_value = mock_octavia_client
        osc.heat.return_value = mock_heat_client

        octavia.delete_loadbalancers(self.context, self.cluster)

        self.assertFalse(mock_octavia_client.load_balancer_show.called)
        self.assertFalse(mock_octavia_client.load_balancer_delete.called)



# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
from oslo_policy import policy


RULE_ADMIN_OR_OWNER = 'rule:admin_or_owner'
RULE_ADMIN_API = 'rule:context_is_admin'
RULE_ADMIN_OR_USER = 'rule:admin_or_user'
RULE_CLUSTER_USER = 'rule:cluster_user'
RULE_DENY_CLUSTER_USER = 'rule:deny_cluster_user'
RULE_USER = "rule:is_user"
# Generic check string for checking if a user is authorized on a particular
# project, specifically with the member role.
RULE_PROJECT_MEMBER = 'rule:project_member'
# Generic check string for checking if a user is authorized on a particular
# project but with read-only access. For example, this persona would be able to
# list private images owned by a project but cannot make any writeable changes
# to those images.
RULE_PROJECT_READER = 'rule:project_reader'

RULE_USER_OR_CLUSTER_USER = (
    'rule:user_or_cluster_user')
RULE_ADMIN_OR_PROJECT_READER = (
    'rule:admin_or_project_reader')
RULE_ADMIN_OR_PROJECT_MEMBER = (
    'rule:admin_or_project_member')
RULE_ADMIN_OR_PROJECT_MEMBER_USER = (
    'rule:admin_or_project_member_user')
RULE_ADMIN_OR_PROJECT_MEMBER_USER_OR_CLUSTER_USER = (
    'rule:admin_or_project_member_user_or_cluster_user')
RULE_PROJECT_MEMBER_DENY_CLUSTER_USER = (
    'rule:project_member_deny_cluster_user')
RULE_ADMIN_OR_PROJECT_MEMBER_DENY_CLUSTER_USER = (
    'rule:admin_or_project_member_deny_cluster_user')
RULE_PROJECT_READER_DENY_CLUSTER_USER = (
    'rule:project_reader_deny_cluster_user')
RULE_ADMIN_OR_PROJECT_READER_DENY_CLUSTER_USER = (
    'rule:admin_or_project_reader_deny_cluster_user')
RULE_ADMIN_OR_PROJECT_READER_USER = (
    'rule:admin_or_project_reader_user')

# ==========================================================
# Deprecated Since OpenStack 2023.2(Magnum 17.0.0) and should be removed in
# The following cycle.

DEPRECATED_REASON = """
The Magnum API now enforces scoped tokens and default reader and member roles.
"""

DEPRECATED_SINCE = 'OpenStack 2023.2(Magnum 17.0.0)'


DEPRECATED_DENY_CLUSTER_USER = policy.DeprecatedRule(
    name=RULE_DENY_CLUSTER_USER,
    check_str='not domain_id:%(trustee_domain_id)s',
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=DEPRECATED_SINCE
)

DEPRECATED_RULE_ADMIN_OR_OWNER = policy.DeprecatedRule(
    name=RULE_ADMIN_OR_OWNER,
    check_str='is_admin:True or project_id:%(project_id)s',
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=DEPRECATED_SINCE
)

# Only used for DEPRECATED_RULE_ADMIN_OR_USER_OR_CLUSTER_USER
RULE_ADMIN_OR_USER_OR_CLUSTER_USER = (
    'rule:admin_or_user_or_cluster_user')

DEPRECATED_RULE_ADMIN_OR_USER_OR_CLUSTER_USER = policy.DeprecatedRule(
    name=RULE_ADMIN_OR_USER_OR_CLUSTER_USER,
    check_str=f"(({RULE_ADMIN_API}) or ({RULE_USER_OR_CLUSTER_USER}))",
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=DEPRECATED_SINCE
)

DEPRECATED_RULE_ADMIN_OR_USER = policy.DeprecatedRule(
    name=RULE_ADMIN_OR_USER,
    check_str=f"(({RULE_ADMIN_API}) or ({RULE_USER}))",
    deprecated_reason=DEPRECATED_REASON,
    deprecated_since=DEPRECATED_SINCE
)
# ==========================================================

rules = [
    policy.RuleDefault(
        name='context_is_admin',
        check_str='role:admin'
    ),
    policy.RuleDefault(
        name='admin_or_owner',
        check_str='is_admin:True or project_id:%(project_id)s'
    ),
    policy.RuleDefault(
        name='admin_or_user',
        check_str='is_admin:True or user_id:%(user_id)s'
    ),
    policy.RuleDefault(
        name='is_user',
        check_str='user_id:%(user_id)s'
    ),
    policy.RuleDefault(
        name='cluster_user',
        check_str='user_id:%(trustee_user_id)s'
    ),
    policy.RuleDefault(
        name='deny_cluster_user',
        check_str='not domain_id:%(trustee_domain_id)s'
    ),
    policy.RuleDefault(
        name='project_member',
        check_str='role:member and project_id:%(project_id)s'
    ),
    policy.RuleDefault(
        name='project_reader',
        check_str='role:reader and project_id:%(project_id)s'
    ),
    policy.RuleDefault(
        name='admin_or_project_reader',
        check_str=f"({RULE_ADMIN_API}) or ({RULE_PROJECT_READER})",
        deprecated_rule=DEPRECATED_RULE_ADMIN_OR_OWNER
    ),
    policy.RuleDefault(
        name='admin_or_project_member',
        check_str=f"({RULE_ADMIN_API}) or ({RULE_PROJECT_MEMBER})",
        deprecated_rule=DEPRECATED_RULE_ADMIN_OR_OWNER
    ),
    policy.RuleDefault(
        name='admin_or_project_member_user',
        check_str=(
            f"({RULE_ADMIN_API}) or (({RULE_PROJECT_MEMBER}) and "
            f"({RULE_USER}))"
        ),
        deprecated_rule=DEPRECATED_RULE_ADMIN_OR_USER
    ),
    policy.RuleDefault(
        name='user_or_cluster_user',
        check_str=(
            f"(({RULE_USER}) or ({RULE_CLUSTER_USER}))"
        )
    ),
    policy.RuleDefault(
        name='admin_or_user_or_cluster_user',
        check_str=(
            f"(({RULE_ADMIN_API}) or ({RULE_USER_OR_CLUSTER_USER}))"
        )
    ),
    policy.RuleDefault(
        name='admin_or_project_member_cluster_user',
        check_str=(
            f"({RULE_ADMIN_API}) or (({RULE_PROJECT_MEMBER}) "
            f"and ({RULE_CLUSTER_USER}))"
        )
    ),
    policy.RuleDefault(
        name='admin_or_project_member_user_or_cluster_user',
        check_str=(
            f"({RULE_ADMIN_API}) or (({RULE_PROJECT_MEMBER}) and "
            f"({RULE_USER_OR_CLUSTER_USER}))"
        ),
        deprecated_rule=DEPRECATED_RULE_ADMIN_OR_USER_OR_CLUSTER_USER
    ),
    policy.RuleDefault(
        name='project_member_deny_cluster_user',
        check_str=(
            f"(({RULE_PROJECT_MEMBER}) and ({RULE_DENY_CLUSTER_USER}))"
        ),
        deprecated_rule=DEPRECATED_DENY_CLUSTER_USER
    ),
    policy.RuleDefault(
        name='admin_or_project_member_deny_cluster_user',
        check_str=(
            f"({RULE_ADMIN_API}) or ({RULE_PROJECT_MEMBER_DENY_CLUSTER_USER})"
        ),
        deprecated_rule=DEPRECATED_DENY_CLUSTER_USER
    ),
    policy.RuleDefault(
        name='project_reader_deny_cluster_user',
        check_str=(
            f"(({RULE_PROJECT_READER}) and ({RULE_DENY_CLUSTER_USER}))"
        ),
        deprecated_rule=DEPRECATED_DENY_CLUSTER_USER
    ),
    policy.RuleDefault(
        name='admin_or_project_reader_deny_cluster_user',
        check_str=(
            f"({RULE_ADMIN_API}) or ({RULE_PROJECT_READER_DENY_CLUSTER_USER})"
        ),
        deprecated_rule=DEPRECATED_DENY_CLUSTER_USER
    ),
    policy.RuleDefault(
        name='admin_or_project_reader_user',
        check_str=(
            f"({RULE_ADMIN_API}) or (({RULE_PROJECT_READER}) and "
            f"({RULE_USER}))"
        ),
        deprecated_rule=DEPRECATED_RULE_ADMIN_OR_USER
    ),
]


def list_rules():
    return rules



# Copyright 2010 United States Government as represented by the
# Administrator of the National Aeronautics and Space Administration.
# All Rights Reserved.
# Copyright 2012 Red Hat, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_middleware import cors

from magnum.common import rpc
import magnum.conf
from magnum import version

CONF = magnum.conf.CONF


def parse_args(argv, default_config_files=None):
    rpc.set_defaults(control_exchange='magnum')
    CONF(argv[1:],
         project='magnum',
         version=version.version_info.release_string(),
         default_config_files=default_config_files)
    rpc.init(CONF)


def set_config_defaults():
    """Update default value for configuration options from other namespace.

    Example, oslo lib config options. This is needed for
    config generator tool to pick these default value changes.
    https://docs.openstack.org/oslo.config/latest/cli/
    generator.html#modifying-defaults-from-other-namespaces
    """
    set_cors_middleware_defaults()


def set_cors_middleware_defaults():
    """Update default configuration options for oslo.middleware."""
    cors.set_defaults(
        allow_headers=['X-Auth-Token',
                       'X-Identity-Status',
                       'X-Roles',
                       'X-Service-Catalog',
                       'X-User-Id',
                       'X-Tenant-Id',
                       'X-OpenStack-Request-ID',
                       'X-Server-Management-Url'],
        expose_headers=['X-Auth-Token',
                        'X-Subject-Token',
                        'X-Service-Token',
                        'X-OpenStack-Request-ID',
                        'X-Server-Management-Url'],
        allow_methods=['GET',
                       'PUT',
                       'POST',
                       'DELETE',
                       'PATCH']
    )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock
from unittest.mock import patch

from oslo_config import cfg

from magnum.common import urlfetch
from magnum.tests import base


class TestUrlFetch(base.BaseTestCase):

    def test_get_unsupported_scheme(self):
        self.assertRaises(urlfetch.URLFetchError,
                          urlfetch.get,
                          'https://example.com',
                          ('http'))

    @patch('requests.get')
    def test_get(self,
                 mock_request_get):
        mock_reader = mock.MagicMock()
        mock_reader.__iter__.return_value = ['a', 'b', 'c']
        mock_response = mock.MagicMock()
        mock_response.iter_content.return_value = mock_reader
        mock_request_get.return_value = mock_response

        self.assertEqual('abc', urlfetch.get('http://example.com'))

    @patch('requests.get')
    def test_get_exceed_manifest_size(self,
                                      mock_request_get):
        cfg.CONF.set_override("max_manifest_size", 1)

        mock_reader = mock.MagicMock()
        mock_reader.__iter__.return_value = ['a', 'b']
        mock_response = mock.MagicMock()
        mock_response.iter_content.return_value = mock_reader
        mock_request_get.return_value = mock_response

        self.assertRaises(urlfetch.URLFetchError,
                          urlfetch.get,
                          'http://example.com')



# Copyright 2015 Huawei Technologies Co.,LTD.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import abc
from oslo_log import log as logging

from magnum.common import exception
from magnum.drivers.common.driver import Driver
from magnum.i18n import _
from magnum import objects


LOG = logging.getLogger(__name__)


def get_scale_manager(context, osclient, cluster):
    cluster_driver = Driver.get_driver_for_cluster(context, cluster)
    manager = cluster_driver.get_scale_manager(context, osclient, cluster)

    # NOTE: Currently only kubernetes cluster scale managers
    # are available.
    return manager


class ScaleManager(object):

    def __init__(self, context, osclient, cluster):
        self.context = context
        self.osclient = osclient
        self.old_cluster = objects.Cluster.get_by_uuid(context, cluster.uuid)
        self.new_cluster = cluster

    def get_removal_nodes(self, hosts_output):
        if not self._is_scale_down():
            return list()

        cluster = self.new_cluster
        stack = self.osclient.heat().stacks.get(cluster.stack_id)
        hosts = hosts_output.get_output_value(stack, cluster)
        if hosts is None:
            raise exception.MagnumException(_(
                "Output key '%(output_key)s' is missing from stack "
                "%(stack_id)s") % {'output_key': hosts_output.heat_output,
                                   'stack_id': stack.id})

        hosts_with_container = self._get_hosts_with_container(self.context,
                                                              cluster)
        hosts_no_container = list(set(hosts) - hosts_with_container)
        LOG.debug('List of hosts that has no container: %s',
                  str(hosts_no_container))

        num_of_removal = self._get_num_of_removal()
        if len(hosts_no_container) < num_of_removal:
            LOG.warning(
                "About to remove %(num_removal)d nodes, which is larger than "
                "the number of empty nodes (%(num_empty)d). %(num_non_empty)d "
                "non-empty nodes will be removed.", {
                    'num_removal': num_of_removal,
                    'num_empty': len(hosts_no_container),
                    'num_non_empty': num_of_removal - len(hosts_no_container)})

        hosts_to_remove = hosts_no_container[0:num_of_removal]
        LOG.info('Require removal of hosts: %s', hosts_to_remove)

        return hosts_to_remove

    def _is_scale_down(self):
        return self.new_cluster.node_count < self.old_cluster.node_count

    def _get_num_of_removal(self):
        return self.old_cluster.node_count - self.new_cluster.node_count

    @abc.abstractmethod
    def _get_hosts_with_container(self, context, cluster):
        """Return the hosts with container running on them."""
        pass



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg
from oslo_config import types

from magnum.common.x509 import extensions
from magnum.i18n import _

ALLOWED_EXTENSIONS = [str(e.value) for e in extensions.Extensions]
DEFAULT_ALLOWED_EXTENSIONS = [
    extensions.Extensions.KEY_USAGE.value,
    extensions.Extensions.EXTENDED_KEY_USAGE.value,
    extensions.Extensions.SUBJECT_ALTERNATIVE_NAME.value,
    extensions.Extensions.BASIC_CONSTRAINTS.value,
    extensions.Extensions.SUBJECT_KEY_IDENTIFIER.value]

ALLOWED_KEY_USAGE = [str(e.value[0]) for e in extensions.KeyUsages]
DEFAULT_ALLOWED_KEY_USAGE = [
    extensions.KeyUsages.DIGITAL_SIGNATURE.value[0],
    extensions.KeyUsages.KEY_ENCIPHERMENT.value[0],
    extensions.KeyUsages.CONTENT_COMMITMENT.value[0]]

x509_group = cfg.OptGroup(name='x509',
                          title='Options for X509 in Magnum')

x509_opts = [
    cfg.BoolOpt('allow_ca',
                default=False,
                help=_('Certificate can get the CA flag in x509 extensions.')),
    cfg.ListOpt('allowed_extensions',
                default=DEFAULT_ALLOWED_EXTENSIONS,
                item_type=types.String(choices=ALLOWED_EXTENSIONS),
                help=_('List of allowed x509 extensions. Available values: '
                       '"%s"') % '", "'.join(ALLOWED_EXTENSIONS)),
    cfg.ListOpt('allowed_key_usage',
                default=DEFAULT_ALLOWED_KEY_USAGE,
                item_type=types.String(choices=ALLOWED_KEY_USAGE),
                help=_('List of allowed x509 key usage. Available values: '
                       '"%s"') % '", "'.join(ALLOWED_KEY_USAGE)),
    cfg.IntOpt('term_of_validity',
               default=365 * 5,
               help=_('Number of days for which a certificate is valid.')),
    cfg.IntOpt('rsa_key_size',
               default=2048, help=_('Size of generated private key.'))]


def register_opts(conf):
    conf.register_group(x509_group)
    conf.register_opts(x509_opts, group=x509_group)


def list_opts():
    return {
        x509_group: x509_opts
    }



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_config import cfg

from magnum.tests.unit.api import base as api_base


CONF = cfg.CONF


class PolicyFunctionalTest(api_base.FunctionalTest):
    def setUp(self):
        super(PolicyFunctionalTest, self).setUp()
        CONF.set_override('enforce_scope', True, group='oslo_policy')
        CONF.set_override('enforce_new_defaults', True, group='oslo_policy')
        self.reader_headers = {
            "X-Roles": "reader",
        }
        self.member_headers = {
            "X-Roles": "member",
        }
        self.admin_headers = {
            "X-Roles": "admin",
        }
        self.foo_headers = {
            "X-Roles": "foo",
        }



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_utils import uuidutils
from webtest.app import AppError

from magnum import objects
from magnum.tests.unit.api import utils as apiutils
from magnum.tests.unit.common.policies import base
from magnum.tests.unit.objects import utils as obj_utils


class TestNodeGroupPolicy(base.PolicyFunctionalTest):
    def setUp(self):
        super(TestNodeGroupPolicy, self).setUp()
        obj_utils.create_test_cluster_template(self.context)
        self.cluster_uuid = uuidutils.generate_uuid()
        obj_utils.create_test_cluster(
            self.context, uuid=self.cluster_uuid)
        self.cluster = objects.Cluster.get_by_uuid(self.context,
                                                   self.cluster_uuid)
        self.nodegroup = obj_utils.create_test_nodegroup(
            self.context, cluster_id=self.cluster.uuid, is_default=False)
        self.url = f"/clusters/{self.cluster.uuid}/nodegroups/"
        self.member = {"Openstack-Api-Version": "container-infra latest"}
        self.member.update(self.member_headers)
        self.reader = {"Openstack-Api-Version": "container-infra latest"}
        self.reader.update(self.reader_headers)

    def test_get_all_no_permission(self):
        exc = self.assertRaises(AppError,
                                self.get_json, self.url,
                                headers=self.member)
        self.assertIn("403 Forbidden", str(exc))

    def test_get_no_permission(self):
        exc = self.assertRaises(
            AppError,
            self.get_json,
            f"{self.url}foo",
            headers=self.member)
        self.assertIn("403 Forbidden", str(exc))

    def test_create_no_permission(self):
        exc = self.assertRaises(AppError,
                                self.post_json, self.url,
                                apiutils.nodegroup_post_data(),
                                headers=self.reader)
        self.assertIn("403 Forbidden", str(exc))

    def test_update_no_permission(self):
        ng_dict = [
            {'path': '/max_node_count', 'value': 4, 'op': 'replace'}]
        exc = self.assertRaises(
            AppError, self.patch_json,
            self.url + self.nodegroup.uuid, ng_dict,
            headers=self.reader)
        self.assertIn("403 Forbidden", str(exc))

    def test_delete_no_permission(self):
        # delete cluster
        exc = self.assertRaises(
                  AppError, self.delete, self.url + self.nodegroup.uuid,
                  headers=self.reader)
        self.assertIn("403 Forbidden", str(exc))



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""add docker_volume_size to cluster

Revision ID: aa0cc27839af
Revises: bc46ba6cf949
Create Date: 2017-06-07 13:08:02.853105

"""

# revision identifiers, used by Alembic.
revision = 'aa0cc27839af'
down_revision = 'bc46ba6cf949'

from alembic import op  # noqa: E402
import sqlalchemy as sa  # noqa: E402


def upgrade():
    pass
    op.add_column('cluster', sa.Column('docker_volume_size',
                                       sa.Integer(), nullable=True))



# Copyright (c) 2012 NTT DOCOMO, INC.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""Magnum DB test base class."""

import fixtures
from oslo_db.sqlalchemy import enginefacade

import magnum.conf
from magnum.db import api as dbapi
from magnum.db.sqlalchemy import migration
from magnum.db.sqlalchemy import models
from magnum.tests import base


CONF = magnum.conf.CONF

_DB_CACHE = None


class Database(fixtures.Fixture):

    def __init__(self, engine, db_migrate, sql_connection):
        self.sql_connection = sql_connection

        self.engine = engine
        self.engine.dispose()
        with self.engine.connect() as conn:
            self.setup_sqlite(db_migrate)
            self.post_migrations()
            self._DB = "".join(line for line in conn.connection.iterdump())
        self.engine.dispose()

    def setup_sqlite(self, db_migrate):
        if db_migrate.version():
            return
        models.Base.metadata.create_all(self.engine)
        db_migrate.stamp('head')

    def setUp(self):
        super(Database, self).setUp()
        with self.engine.connect() as conn:
            conn.connection.executescript(self._DB)
        self.addCleanup(self.engine.dispose)

    def post_migrations(self):
        """Any addition steps that are needed outside of the migrations."""


class DbTestCase(base.TestCase):

    def setUp(self):
        super(DbTestCase, self).setUp()

        self.dbapi = dbapi.get_instance()

        global _DB_CACHE
        if not _DB_CACHE:
            engine = enginefacade.writer.get_engine()
            _DB_CACHE = Database(engine, migration,
                                 sql_connection=CONF.database.connection)
            engine.dispose()
        self.useFixture(_DB_CACHE)



# Copyright 2015 EasyStack, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from glanceclient import exc as glance_exception
from novaclient import exceptions as nova_exception

from magnum.api import utils as api_utils
from magnum.common import clients
from magnum.common import exception
from magnum.i18n import _


SUPPORTED_ISOLATION = ['filesystem/posix', 'filesystem/linux',
                       'filesystem/shared', 'posix/cpu',
                       'posix/mem', 'posix/disk', 'cgroups/cpu',
                       'cgroups/mem', 'docker/runtime',
                       'namespaces/pid']
SUPPORTED_IMAGE_PROVIDERS = ['docker', 'appc']


def validate_image(cli, image):
    """Validate image"""

    try:
        image_found = api_utils.get_openstack_resource(cli.glance().images,
                                                       image, 'images')
    except (glance_exception.NotFound, exception.ResourceNotFound):
        raise exception.ImageNotFound(image_id=image)
    except glance_exception.HTTPForbidden:
        raise exception.ImageNotAuthorized(image_id=image)
    if not image_found.get('os_distro'):
        raise exception.OSDistroFieldNotFound(image_id=image)
    return image_found


def validate_flavor(cli, flavor):
    """Validate flavor.

    If flavor is None, skip the validation and use the default value
    from the heat template.
    """

    if flavor is None:
        return
    flavor_list = cli.nova().flavors.list()
    for f in flavor_list:
        if f.name == flavor or f.id == flavor:
            return
    raise exception.FlavorNotFound(flavor=flavor)


def validate_keypair(cli, keypair):
    """Validate keypair

    validate the keypair, if provided.
    """
    if keypair is None:
        return
    try:
        cli.nova().keypairs.get(keypair)
    except nova_exception.NotFound:
        raise exception.KeyPairNotFound(keypair=keypair)


def validate_external_network(cli, external_network):
    """Validate external network"""

    count = 0
    ext_filter = {'router:external': True}
    networks = cli.neutron().list_networks(**ext_filter)
    for net in networks.get('networks'):
        if (net.get('name') == external_network or
                net.get('id') == external_network):
            count = count + 1

    if count == 0:
        # Unable to find the external network.
        # Or the network is private.
        raise exception.ExternalNetworkNotFound(network=external_network)

    if count > 1:
        msg = _("Multiple external networks exist with same name '%s'. "
                "Please use the external network ID instead.")
        raise exception.Conflict(msg % external_network)


def validate_fixed_network(cli, fixed_network):
    """Validate fixed network"""

    count = 0
    network_id = None
    networks = cli.neutron().list_networks()
    for net in networks.get('networks'):
        if fixed_network in [net.get('name'), net.get('id')]:
            count += 1
            network_id = net.get('id')

    if count == 0:
        # Unable to find the configured fixed_network.
        raise exception.FixedNetworkNotFound(network=fixed_network)
    elif count > 1:
        msg = _("Multiple networks exist with same name '%s'. "
                "Please use the network ID instead.")
        raise exception.Conflict(msg % fixed_network)

    return network_id


def validate_fixed_subnet(cli, fixed_subnet):
    """Validate fixed subnet"""

    count = 0
    subnet_id = None
    subnets = cli.neutron().list_subnets()
    for subnet in subnets.get('subnets'):
        if fixed_subnet in [subnet.get('name'), subnet.get('id')]:
            count += 1
            subnet_id = subnet.get('id')

    if count == 0:
        # Unable to find the configured fixed_subnet.
        raise exception.FixedSubnetNotFound(subnet=fixed_subnet)
    elif count > 1:
        msg = _("Multiple subnets exist with same name '%s'. "
                "Please use the subnet ID instead.")
        raise exception.Conflict(msg % fixed_subnet)
    else:
        return subnet_id


def validate_labels(labels):
    """"Validate labels"""

    for attr, validate_method in labels_validators.items():
        if labels.get(attr) is not None:
            validate_method(labels)


def validate_os_resources(context, cluster_template, cluster=None):
    """Validate ClusterTemplate's OpenStack Resources"""

    cli = clients.OpenStackClients(context)

    for attr, validate_method in validators.items():
        if cluster and attr in cluster and cluster[attr]:
            if attr == 'labels':
                validate_method(cluster[attr])
            else:
                validate_method(cli, cluster[attr])
        elif attr in cluster_template and cluster_template[attr] is not None:
            if attr == 'labels':
                validate_method(cluster_template[attr])
            else:
                validate_method(cli, cluster_template[attr])

    if cluster:
        validate_keypair(cli, cluster['keypair'])


def validate_master_count(context, cluster):
    if (
        cluster['master_count'] > 1 and
        not cluster['master_lb_enabled']
    ):
        raise exception.InvalidParameterValue(_(
            "master_count must be 1 when master_lb_enabled is False"))


def validate_federation_hostcluster(cluster_uuid):
    """Validate Federation `hostcluster_id` parameter.

    If the parameter was not specified raise an
    `exceptions.InvalidParameterValue`. If the specified identifier does not
    identify any Cluster, raise `exception.ClusterNotFound`
    """
    if cluster_uuid is not None:
        api_utils.get_resource('Cluster', cluster_uuid)
    else:
        raise exception.InvalidParameterValue(
            "No hostcluster specified. "
            "Please specify a hostcluster_id.")


def validate_federation_properties(properties):
    """Validate Federation `properties` parameter."""
    if properties is None:
        raise exception.InvalidParameterValue(
            "Please specify a `properties` "
            "dict for the federation.")
    # Currently, we only support the property `dns-zone`.
    if properties.get('dns-zone') is None:
        raise exception.InvalidParameterValue("No DNS zone specified. "
                                              "Please specify a `dns-zone`.")


# Dictionary that maintains a list of validation functions
validators = {'image_id': validate_image,
              'flavor_id': validate_flavor,
              'master_flavor_id': validate_flavor,
              'external_network_id': validate_external_network,
              'fixed_network': validate_fixed_network,
              'fixed_subnet': validate_fixed_subnet,
              'labels': validate_labels}

labels_validators = {}



#
#  Licensed under the Apache License, Version 2.0 (the "License"); you may
#  not use this file except in compliance with the License. You may obtain
#  a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#  Unless required by applicable law or agreed to in writing, software
#  distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#  WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#  License for the specific language governing permissions and limitations
#  under the License.

"""add driver to cluster_template

Revision ID: c0f832afc4fd
Revises: 7da8489d6a68
Create Date: 2024-01-29 13:18:15.181043

"""

# revision identifiers, used by Alembic.
revision = 'c0f832afc4fd'
down_revision = '7da8489d6a68'

from alembic import op  # noqa: E402
import sqlalchemy as sa  # noqa: E402


def upgrade():
    op.add_column('cluster_template',
                  sa.Column('driver', sa.String(length=255),
                            nullable=True))
    # ### end Alembic commands ###



# Copyright 2015 OpenStack Foundation
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from unittest import mock

from oslo_utils import uuidutils
from testtools.matchers import HasLength

from magnum.common import exception
from magnum import objects
from magnum.tests.unit.db import base
from magnum.tests.unit.db import utils


class TestClusterTemplateObject(base.DbTestCase):

    def setUp(self):
        super(TestClusterTemplateObject, self).setUp()
        self.fake_cluster_template = utils.get_test_cluster_template()

    def test_get_by_id(self):
        cluster_template_id = self.fake_cluster_template['id']
        with mock.patch.object(self.dbapi, 'get_cluster_template_by_id',
                               autospec=True) as mock_get_cluster_template:
            mock_get_cluster_template.return_value = self.fake_cluster_template
            cluster_template = objects.ClusterTemplate.get(self.context,
                                                           cluster_template_id)
            mock_get_cluster_template.assert_called_once_with(
                self.context, cluster_template_id)
            self.assertEqual(self.context, cluster_template._context)

    def test_get_by_uuid(self):
        uuid = self.fake_cluster_template['uuid']
        with mock.patch.object(self.dbapi, 'get_cluster_template_by_uuid',
                               autospec=True) as mock_get_cluster_template:
            mock_get_cluster_template.return_value = self.fake_cluster_template
            cluster_template = objects.ClusterTemplate.get(self.context, uuid)
            mock_get_cluster_template.assert_called_once_with(self.context,
                                                              uuid)
            self.assertEqual(self.context, cluster_template._context)

    def test_get_bad_id_and_uuid(self):
        self.assertRaises(exception.ClusterTemplateNotFound,
                          objects.ClusterTemplate.get, self.context,
                          'not-a-uuid')

    def test_get_by_name(self):
        name = self.fake_cluster_template['name']
        with mock.patch.object(self.dbapi, 'get_cluster_template_by_name',
                               autospec=True) as mock_get_cluster_template:
            mock_get_cluster_template.return_value = self.fake_cluster_template
            cluster_template = objects.ClusterTemplate.get_by_name(
                self.context, name)
            mock_get_cluster_template.assert_called_once_with(self.context,
                                                              name)
            self.assertEqual(self.context, cluster_template._context)

    def test_list(self):
        with mock.patch.object(self.dbapi, 'get_cluster_template_list',
                               autospec=True) as mock_get_list:
            mock_get_list.return_value = [self.fake_cluster_template]
            cluster_templates = objects.ClusterTemplate.list(self.context)
            self.assertEqual(1, mock_get_list.call_count)
            self.assertThat(cluster_templates, HasLength(1))
            self.assertIsInstance(cluster_templates[0],
                                  objects.ClusterTemplate)
            self.assertEqual(self.context, cluster_templates[0]._context)

    def test_create(self):
        with mock.patch.object(self.dbapi, 'create_cluster_template',
                               autospec=True) as mock_create_cluster_template:
            mock_create_cluster_template.return_value = \
                self.fake_cluster_template
            cluster_template = objects.ClusterTemplate(
                self.context, **self.fake_cluster_template)
            cluster_template.create()
            mock_create_cluster_template.assert_called_once_with(
                self.fake_cluster_template)
            self.assertEqual(self.context, cluster_template._context)

    def test_destroy(self):
        uuid = self.fake_cluster_template['uuid']
        with mock.patch.object(self.dbapi, 'get_cluster_template_by_uuid',
                               autospec=True) as mock_get_cluster_template:
            mock_get_cluster_template.return_value = self.fake_cluster_template
            with mock.patch.object(
                    self.dbapi, 'destroy_cluster_template', autospec=True)\
                    as mock_destroy_cluster_template:
                cluster_template = objects.ClusterTemplate.get_by_uuid(
                    self.context, uuid)
                cluster_template.destroy()
                mock_get_cluster_template.assert_called_once_with(self.context,
                                                                  uuid)
                mock_destroy_cluster_template.assert_called_once_with(uuid)
                self.assertEqual(self.context, cluster_template._context)

    def test_save(self):
        uuid = self.fake_cluster_template['uuid']
        with mock.patch.object(self.dbapi, 'get_cluster_template_by_uuid',
                               autospec=True) as mock_get_cluster_template:
            mock_get_cluster_template.return_value = self.fake_cluster_template
            with mock.patch.object(self.dbapi, 'update_cluster_template',
                                   autospec=True) \
                    as mock_update_cluster_template:
                cluster_template = objects.ClusterTemplate.get_by_uuid(
                    self.context, uuid)
                cluster_template.image_id = 'test-image'
                cluster_template.save()

                mock_get_cluster_template.assert_called_once_with(self.context,
                                                                  uuid)
                mock_update_cluster_template.assert_called_once_with(
                    uuid, {'image_id': 'test-image'})
                self.assertEqual(self.context, cluster_template._context)

    def test_refresh(self):
        uuid = self.fake_cluster_template['uuid']
        new_uuid = uuidutils.generate_uuid()
        returns = [dict(self.fake_cluster_template, uuid=uuid),
                   dict(self.fake_cluster_template, uuid=new_uuid)]
        expected = [mock.call(self.context, uuid),
                    mock.call(self.context, uuid)]
        with mock.patch.object(self.dbapi, 'get_cluster_template_by_uuid',
                               side_effect=returns,
                               autospec=True) as mock_get_cluster_template:
            cluster_template = objects.ClusterTemplate.get_by_uuid(
                self.context, uuid)
            self.assertEqual(uuid, cluster_template.uuid)
            cluster_template.refresh()
            self.assertEqual(new_uuid, cluster_template.uuid)
            self.assertEqual(expected,
                             mock_get_cluster_template.call_args_list)
            self.assertEqual(self.context, cluster_template._context)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import base64
from oslo_log import log as logging
from oslo_utils import strutils

from magnum.common import utils
from magnum.common.x509 import operations as x509
from magnum.conductor.handlers.common import cert_manager
from magnum.drivers.heat import k8s_template_def
from magnum.drivers.heat import template_def
from oslo_config import cfg

CONF = cfg.CONF

LOG = logging.getLogger(__name__)


class CoreOSK8sTemplateDefinition(k8s_template_def.K8sTemplateDefinition):
    """Kubernetes template for a CoreOS."""

    def __init__(self):
        super(CoreOSK8sTemplateDefinition, self).__init__()
        self.add_parameter('docker_storage_driver',
                           cluster_template_attr='docker_storage_driver')

    def get_params(self, context, cluster_template, cluster, **kwargs):
        extra_params = kwargs.pop('extra_params', {})

        extra_params['username'] = context.user_name
        osc = self.get_osc(context)
        extra_params['region_name'] = osc.cinder_region_name()

        # set docker_volume_type
        # use the configuration default if None provided
        docker_volume_type = cluster.labels.get(
            'docker_volume_type', CONF.cinder.default_docker_volume_type)
        extra_params['docker_volume_type'] = docker_volume_type

        extra_params['nodes_affinity_policy'] = \
            CONF.cluster.nodes_affinity_policy

        if cluster_template.network_driver == 'flannel':
            extra_params["pods_network_cidr"] = \
                cluster.labels.get('flannel_network_cidr', '10.100.0.0/16')
        if cluster_template.network_driver == 'calico':
            extra_params["pods_network_cidr"] = \
                cluster.labels.get('calico_ipv4pool', '10.100.0.0/16')

        label_list = ['coredns_tag',
                      'kube_tag', 'container_infra_prefix',
                      'availability_zone',
                      'calico_tag',
                      'calico_ipv4pool',
                      'calico_ipv4pool_ipip',
                      'etcd_tag', 'flannel_tag']

        labels = self._get_relevant_labels(cluster, kwargs)

        for label in label_list:
            label_value = labels.get(label)
            if label_value:
                extra_params[label] = label_value

        cert_manager_api = cluster.labels.get('cert_manager_api')
        if strutils.bool_from_string(cert_manager_api):
            extra_params['cert_manager_api'] = cert_manager_api
            ca_cert = cert_manager.get_cluster_ca_certificate(cluster)
            extra_params['ca_key'] = x509.decrypt_key(
                ca_cert.get_private_key(),
                ca_cert.get_private_key_passphrase()).replace("\n", "\\n")

        plain_openstack_ca = utils.get_openstack_ca()
        encoded_openstack_ca = base64.b64encode(plain_openstack_ca.encode())
        extra_params['openstack_ca_coreos'] = encoded_openstack_ca.decode()

        return super(CoreOSK8sTemplateDefinition,
                     self).get_params(context, cluster_template, cluster,
                                      extra_params=extra_params,
                                      **kwargs)

    def get_env_files(self, cluster_template, cluster, nodegroup=None):
        env_files = []

        template_def.add_priv_net_env_file(env_files, cluster_template,
                                           cluster)
        template_def.add_etcd_volume_env_file(env_files, cluster)
        template_def.add_volume_env_file(env_files, cluster,
                                         nodegroup=nodegroup)
        template_def.add_lb_env_file(env_files, cluster)
        template_def.add_fip_env_file(env_files, cluster)

        return env_files



# Copyright (c) 2015 Intel Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import functools

from oslo_log import log
from oslo_service import loopingcall
from oslo_service import periodic_task

from pycadf import cadftaxonomy as taxonomy

from magnum.common import clients
from magnum.common import context
from magnum.common import exception
from magnum.common import profiler
from magnum.common import rpc
from magnum.conductor.handlers.common import cert_manager
from magnum.conductor.handlers.common import trust_manager
from magnum.conductor import monitors
from magnum.conductor import utils as conductor_utils
import magnum.conf
from magnum.drivers.common import driver
from magnum import objects


CONF = magnum.conf.CONF
LOG = log.getLogger(__name__)


def set_context(func):
    @functools.wraps(func)
    def handler(self, ctx):
        ctx = context.make_admin_context(all_tenants=True)
        context.set_ctx(ctx)
        func(self, ctx)
        context.set_ctx(None)
    return handler


class ClusterUpdateJob(object):

    status_to_event = {
        objects.fields.ClusterStatus.DELETE_COMPLETE: taxonomy.ACTION_DELETE,
        objects.fields.ClusterStatus.CREATE_COMPLETE: taxonomy.ACTION_CREATE,
        objects.fields.ClusterStatus.UPDATE_COMPLETE: taxonomy.ACTION_UPDATE,
        objects.fields.ClusterStatus.ROLLBACK_COMPLETE: taxonomy.ACTION_UPDATE,
        objects.fields.ClusterStatus.CREATE_FAILED: taxonomy.ACTION_CREATE,
        objects.fields.ClusterStatus.DELETE_FAILED: taxonomy.ACTION_DELETE,
        objects.fields.ClusterStatus.UPDATE_FAILED: taxonomy.ACTION_UPDATE,
        objects.fields.ClusterStatus.ROLLBACK_FAILED: taxonomy.ACTION_UPDATE
    }

    def __init__(self, ctx, cluster):
        self.ctx = ctx
        self.cluster = cluster

    def update_status(self):
        LOG.debug("Updating status for cluster %s", self.cluster.id)
        # get the driver for the cluster
        cdriver = driver.Driver.get_driver_for_cluster(self.ctx, self.cluster)
        # ask the driver to sync status
        try:
            cdriver.update_cluster_status(self.ctx, self.cluster)
        except exception.AuthorizationFailure as e:
            trust_ex = ("Could not find trust: %s" % self.cluster.trust_id)
            # Try to use admin context if trust not found.
            # This will make sure even with trust got deleted out side of
            # Magnum, we still be able to check cluster status
            if trust_ex in str(e):
                cdriver.update_cluster_status(
                    self.ctx, self.cluster, use_admin_ctx=True)
            else:
                raise

        LOG.debug("Status for cluster %s updated to %s (%s)",
                  self.cluster.id, self.cluster.status,
                  self.cluster.status_reason)
        # status update notifications
        if self.cluster.status.endswith("_COMPLETE"):
            conductor_utils.notify_about_cluster_operation(
                self.ctx, self.status_to_event[self.cluster.status],
                taxonomy.OUTCOME_SUCCESS, self.cluster)
        if self.cluster.status.endswith("_FAILED"):
            conductor_utils.notify_about_cluster_operation(
                self.ctx, self.status_to_event[self.cluster.status],
                taxonomy.OUTCOME_FAILURE, self.cluster)
        # if we're done with it, delete it
        if self.cluster.status == objects.fields.ClusterStatus.DELETE_COMPLETE:
            # Clean up trusts and certificates, if they still exist.
            os_client = clients.OpenStackClients(self.ctx)
            LOG.debug("Calling delete_trustee_and_trusts from periodic "
                      "DELETE_COMPLETE")
            trust_manager.delete_trustee_and_trust(os_client, self.ctx,
                                                   self.cluster)
            cert_manager.delete_certificates_from_cluster(self.cluster,
                                                          context=self.ctx)
            # delete all the nodegroups that belong to this cluster
            for ng in objects.NodeGroup.list(self.ctx, self.cluster.uuid):
                ng.destroy()
            self.cluster.destroy()
        # end the "loop"
        raise loopingcall.LoopingCallDone()


class ClusterHealthUpdateJob(object):

    def __init__(self, ctx, cluster):
        self.ctx = ctx
        self.cluster = cluster

    def _update_health_status(self):
        monitor = monitors.create_monitor(self.ctx, self.cluster)
        if monitor is None:
            return

        try:
            monitor.poll_health_status()
        except Exception as e:
            LOG.warning(
                "Skip pulling data from cluster %(cluster)s due to "
                "error: %(e)s",
                {'e': e, 'cluster': self.cluster.uuid}, exc_info=True)
            # TODO(flwang): Should we mark this cluster's health status as
            # UNKNOWN if Magnum failed to pull data from the cluster? Because
            # that basically means the k8s API doesn't work at that moment.
            return

        if monitor.data.get('health_status'):
            self.cluster.health_status = monitor.data.get('health_status')
            self.cluster.health_status_reason = monitor.data.get(
                'health_status_reason')
            self.cluster.save()

    def update_health_status(self):
        LOG.debug("Updating health status for cluster %s", self.cluster.id)
        self._update_health_status()
        LOG.debug("Status for cluster %s updated to %s (%s)",
                  self.cluster.id, self.cluster.health_status,
                  self.cluster.health_status_reason)
        # TODO(flwang): Health status update notifications?
        # end the "loop"
        raise loopingcall.LoopingCallDone()


@profiler.trace_cls("rpc")
class MagnumPeriodicTasks(periodic_task.PeriodicTasks):
    """Magnum periodic Task class

    Any periodic task job need to be added into this class

    NOTE(suro-patz):
    - oslo_service.periodic_task runs tasks protected within try/catch
      block, with default raise_on_error as 'False', in run_periodic_tasks(),
      which ensures the process does not die, even if a task encounters an
      Exception.
    - The periodic tasks here does not necessarily need another
      try/catch block. The present try/catch block here helps putting
      magnum-periodic-task-specific log/error message.

    """

    def __init__(self, conf):
        super(MagnumPeriodicTasks, self).__init__(conf)
        self.notifier = rpc.get_notifier()

    @periodic_task.periodic_task(spacing=10, run_immediately=True)
    @set_context
    def sync_cluster_status(self, ctx):
        try:
            LOG.debug('Starting to sync up cluster status')

            # get all the clusters that are IN_PROGRESS
            status = [objects.fields.ClusterStatus.CREATE_IN_PROGRESS,
                      objects.fields.ClusterStatus.UPDATE_IN_PROGRESS,
                      objects.fields.ClusterStatus.DELETE_IN_PROGRESS,
                      objects.fields.ClusterStatus.ROLLBACK_IN_PROGRESS]
            filters = {'status': status}
            clusters = objects.Cluster.list(ctx, filters=filters)
            if not clusters:
                return

            # synchronize with underlying orchestration
            for cluster in clusters:
                job = ClusterUpdateJob(ctx, cluster)
                # though this call isn't really looping, we use this
                # abstraction anyway to avoid dealing directly with eventlet
                # hooey
                lc = loopingcall.FixedIntervalLoopingCall(f=job.update_status)
                lc.start(1, stop_on_exception=True)

        except Exception as e:
            LOG.warning(
                "Ignore error [%s] when syncing up cluster status.",
                e, exc_info=True)

    @periodic_task.periodic_task(
        spacing=CONF.kubernetes.health_polling_interval,
        run_immediately=True)
    @set_context
    def sync_cluster_health_status(self, ctx):
        try:
            LOG.debug('Starting to sync up cluster health status')

            status = [objects.fields.ClusterStatus.CREATE_COMPLETE,
                      objects.fields.ClusterStatus.UPDATE_COMPLETE,
                      objects.fields.ClusterStatus.UPDATE_IN_PROGRESS,
                      objects.fields.ClusterStatus.ROLLBACK_IN_PROGRESS]
            filters = {'status': status}
            clusters = objects.Cluster.list(ctx, filters=filters)
            if not clusters:
                return

            # synchronize using native COE API
            for cluster in clusters:
                job = ClusterHealthUpdateJob(ctx, cluster)
                # though this call isn't really looping, we use this
                # abstraction anyway to avoid dealing directly with eventlet
                # hooey
                lc = loopingcall.FixedIntervalLoopingCall(
                    f=job.update_health_status)
                lc.start(1, stop_on_exception=True)

        except Exception as e:
            LOG.warning(
                "Ignore error [%s] when syncing up cluster status.",
                e, exc_info=True)


def setup(conf, tg):
    pt = MagnumPeriodicTasks(conf)
    tg.add_dynamic_timer(
        pt.run_periodic_tasks,
        periodic_interval_max=conf.periodic_interval_max,
        context=None)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

import random
import socket
import string
import struct

from tempest.lib.common.utils import data_utils

from magnum.tests.functional.api.v1.models import cert_model
from magnum.tests.functional.api.v1.models import cluster_model
from magnum.tests.functional.api.v1.models import cluster_template_model
from magnum.tests.functional.api.v1.models import cluster_templatepatch_model
from magnum.tests.functional.api.v1.models import clusterpatch_model
from magnum.tests.functional.common import config


def random_int(min_int=1, max_int=100):
    return random.randrange(min_int, max_int)


def gen_coe_dep_network_driver(coe):
    allowed_driver_types = {
        'kubernetes': ['flannel', None],
    }
    driver_types = allowed_driver_types[coe]
    return driver_types[random.randrange(0, len(driver_types))]


def gen_coe_dep_volume_driver(coe):
    allowed_driver_types = {
        'kubernetes': ['cinder', None],
    }
    driver_types = allowed_driver_types[coe]
    return driver_types[random.randrange(0, len(driver_types))]


def gen_random_port():
    return random_int(49152, 65535)


def gen_docker_volume_size(min_int=3, max_int=5):
    return random_int(min_int, max_int)


def gen_fake_ssh_pubkey():
    chars = "".join(
        random.choice(string.ascii_uppercase +
                      string.ascii_letters + string.digits + '/+=')
        for _ in range(372))
    return "ssh-rsa " + chars


def gen_random_ip():
    return socket.inet_ntoa(struct.pack('>I', random.randint(1, 0xffffffff)))


def gen_url(scheme="http", domain="example.com", port=80):
    return "%s://%s:%s" % (scheme, domain, port)


def gen_http_proxy():
    return gen_url(port=gen_random_port())


def gen_https_proxy():
    return gen_url(scheme="https", port=gen_random_port())


def gen_no_proxy():
    return ",".join(gen_random_ip() for x in range(3))


def cert_data(cluster_uuid, csr_data):
    data = {
        "cluster_uuid": cluster_uuid,
        "csr": csr_data}

    model = cert_model.CertEntity.from_dict(data)

    return model


def cluster_template_data(**kwargs):
    """Generates random cluster_template data

    Keypair and image id cannot be random for the cluster_template to be valid
    due to validations for the presence of keypair and image id prior to
    cluster_template creation.

    :param keypair_id: keypair name
    :param image_id: image id or name
    :returns: ClusterTemplateEntity with generated data
    """

    data = {
        "name": data_utils.rand_name('cluster'),
        "coe": "kubernetes",
        "tls_disabled": False,
        "network_driver": None,
        "volume_driver": None,
        "labels": {},
        "public": False,
        "dns_nameserver": "8.8.8.8",
        "flavor_id": data_utils.rand_name('cluster'),
        "master_flavor_id": data_utils.rand_name('cluster'),
        "external_network_id": config.Config.nic_id,
        "keypair_id": data_utils.rand_name('cluster'),
        "image_id": data_utils.rand_name('cluster')
    }

    data.update(kwargs)
    model = cluster_template_model.ClusterTemplateEntity.from_dict(data)

    return model


def cluster_template_replace_patch_data(path,
                                        value=data_utils.rand_name('cluster')):
    """Generates random ClusterTemplate patch data

    :param path: path to replace
    :param value: value to replace in patch
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": path,
        "value": value,
        "op": "replace"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_remove_patch_data(path):
    """Generates ClusterTemplate patch data by removing value

    :param path: path to remove
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": path,
        "op": "remove"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_name_patch_data(name=data_utils.rand_name('cluster')):
    """Generates random cluster_template patch data

    :param name: name to replace in patch
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": "/name",
        "value": name,
        "op": "replace"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_flavor_patch_data(flavor=data_utils.rand_name('cluster')):
    """Generates random cluster_template patch data

    :param flavor: flavor to replace in patch
    :returns: ClusterTemplatePatchCollection with generated data
    """

    data = [{
        "path": "/flavor_id",
        "value": flavor,
        "op": "replace"
    }]
    collection = cluster_templatepatch_model.ClusterTemplatePatchCollection
    return collection.from_dict(data)


def cluster_template_data_with_valid_keypair_image_flavor():
    """Generates random clustertemplate data with valid data

    :returns: ClusterTemplateEntity with generated data
    """
    master_flavor = config.Config.master_flavor_id
    return cluster_template_data(keypair_id=config.Config.keypair_id,
                                 image_id=config.Config.image_id,
                                 flavor_id=config.Config.flavor_id,
                                 master_flavor_id=master_flavor)


def cluster_template_data_with_missing_image():
    """Generates random cluster_template data with missing image

    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(
        keypair_id=config.Config.keypair_id,
        flavor_id=config.Config.flavor_id,
        master_flavor_id=config.Config.master_flavor_id)


def cluster_template_data_with_missing_flavor():
    """Generates random cluster_template data with missing flavor

    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(keypair_id=config.Config.keypair_id,
                                 image_id=config.Config.image_id)


def cluster_template_data_with_missing_keypair():
    """Generates random cluster_template data with missing keypair

    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(
        image_id=config.Config.image_id,
        flavor_id=config.Config.flavor_id,
        master_flavor_id=config.Config.master_flavor_id)


def cluster_template_valid_data_with_specific_coe(coe):
    """Generates random cluster_template data with valid keypair and image

    :param coe: coe
    :returns: ClusterTemplateEntity with generated data
    """

    return cluster_template_data(keypair_id=config.Config.keypair_id,
                                 image_id=config.Config.image_id, coe=coe)


def cluster_data(name=data_utils.rand_name('cluster'),
                 cluster_template_id=data_utils.rand_uuid(),
                 node_count=random_int(1, 5), discovery_url=gen_random_ip(),
                 create_timeout=random_int(1, 30),
                 master_count=random_int(1, 5)):
    """Generates random cluster data

    cluster_template_id cannot be random for the cluster to be valid due to
    validations for the presence of clustertemplate prior to clustertemplate
    creation.

    :param name: cluster name (must be unique)
    :param cluster_template_id: clustertemplate unique id (must already exist)
    :param node_count: number of agents for cluster
    :param discovery_url: url provided for node discovery
    :param create_timeout: timeout in minutes for cluster create
    :param master_count: number of master nodes for the cluster
    :returns: ClusterEntity with generated data
    """

    data = {
        "name": name,
        "cluster_template_id": cluster_template_id,
        "keypair": config.Config.keypair_id,
        "node_count": node_count,
        "discovery_url": None,
        "create_timeout": create_timeout,
        "master_count": master_count
    }
    model = cluster_model.ClusterEntity.from_dict(data)

    return model


def valid_cluster_data(cluster_template_id,
                       name=data_utils.rand_name('cluster'),
                       node_count=1, master_count=1, create_timeout=None):
    """Generates random cluster data with valid

    :param cluster_template_id: clustertemplate unique id that already exists
    :param name: cluster name (must be unique)
    :param node_count: number of agents for cluster
    :returns: ClusterEntity with generated data
    """

    return cluster_data(cluster_template_id=cluster_template_id, name=name,
                        master_count=master_count, node_count=node_count,
                        create_timeout=create_timeout)


def cluster_name_patch_data(name=data_utils.rand_name('cluster')):
    """Generates random clustertemplate patch data

    :param name: name to replace in patch
    :returns: ClusterPatchCollection with generated data
    """

    data = [{
        "path": "/name",
        "value": name,
        "op": "replace"
    }]
    return clusterpatch_model.ClusterPatchCollection.from_dict(data)


def cluster_api_addy_patch_data(address='0.0.0.0'):
    """Generates random cluster patch data

    :param name: name to replace in patch
    :returns: ClusterPatchCollection with generated data
    """

    data = [{
        "path": "/api_address",
        "value": address,
        "op": "replace"
    }]
    return clusterpatch_model.ClusterPatchCollection.from_dict(data)


def cluster_node_count_patch_data(node_count=2):
    """Generates random cluster patch data

    :param name: name to replace in patch
    :returns: ClusterPatchCollection with generated data
    """

    data = [{
        "path": "/node_count",
        "value": node_count,
        "op": "replace"
    }]
    return clusterpatch_model.ClusterPatchCollection.from_dict(data)



# Copyright (c) 2016 Intel, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from magnum.common.cert_manager import cert_manager
from magnum import objects


class Cert(cert_manager.Cert):
    """Representation of a Cert for Magnum DB storage."""
    def __init__(self, certificate, private_key, intermediates=None,
                 private_key_passphrase=None):
        self.certificate = certificate
        self.intermediates = intermediates
        self.private_key = private_key
        self.private_key_passphrase = private_key_passphrase

    def get_certificate(self):
        return self.certificate

    def get_intermediates(self):
        return self.intermediates

    def get_private_key(self):
        return self.private_key

    def get_private_key_passphrase(self):
        return self.private_key_passphrase


class CertManager(cert_manager.CertManager):
    """Cert Manager Interface that stores data locally in Magnum db.

    """

    @staticmethod
    def store_cert(certificate, private_key, intermediates=None,
                   private_key_passphrase=None, context=None, **kwargs):
        """Stores (i.e., registers) a cert with the cert manager.

        This method stores the specified cert to x509keypair model and returns
        a UUID that can be used to retrieve it.

        :param certificate: PEM encoded TLS certificate
        :param private_key: private key for the supplied certificate
        :param intermediates: ordered and concatenated intermediate certs
        :param private_key_passphrase: optional passphrase for the supplied key

        :returns: the UUID of the stored cert
        """
        if isinstance(certificate, bytes):
            certificate = certificate.decode()
        if isinstance(private_key, bytes):
            private_key = private_key.decode()
        x509keypair = {'certificate': certificate, 'private_key': private_key,
                       'private_key_passphrase': private_key_passphrase,
                       'intermediates': intermediates,
                       'project_id': context.project_id,
                       'user_id': context.user_id}
        x509keypair_obj = objects.X509KeyPair(context, **x509keypair)
        x509keypair_obj.create()
        return x509keypair_obj.uuid

    @staticmethod
    def get_cert(cert_ref, context=None, **kwargs):
        """Retrieves the specified cert.

        :param cert_ref: the UUID of the cert to retrieve

        :return: magnum.common.cert_manager.cert_manager.Cert
                 representation of the certificate data
        """
        cert_data = dict()
        x509keypair_obj = objects.X509KeyPair.get_by_uuid(context, cert_ref)
        cert_data['certificate'] = x509keypair_obj.certificate
        cert_data['private_key'] = x509keypair_obj.private_key
        cert_data['private_key_passphrase'] = \
            x509keypair_obj.private_key_passphrase
        cert_data['intermediates'] = x509keypair_obj.intermediates
        return Cert(**cert_data)

    @staticmethod
    def delete_cert(cert_ref, context=None, **kwargs):
        """Deletes the specified cert.

        :param cert_ref: the UUID of the cert to delete
        """
        x509keypair_obj = objects.X509KeyPair.get_by_uuid(context, cert_ref)
        x509keypair_obj.destroy()



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import datetime
from unittest import mock

from oslo_utils import timeutils

from magnum.api import servicegroup as svc_grp
from magnum.tests.unit.api import base as api_base
from magnum.tests.unit.objects import utils as obj_util


class TestServiceGroup(api_base.FunctionalTest):
    def setUp(self):
        super(TestServiceGroup, self).setUp()
        self.servicegroup_api = svc_grp.ServiceGroup()

    def test_service_is_up_check_type(self):
        random_obj = mock.MagicMock()
        self.assertRaises(TypeError,
                          self.servicegroup_api.service_is_up, random_obj)

    def test_service_is_up_forced_down(self):
        kwarg = {'forced_down': True}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertFalse(is_up)

    def test_service_is_up_alive(self):
        kwarg = {'last_seen_up': timeutils.utcnow(True)}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertTrue(is_up)

    def test_service_is_up_alive_with_created(self):
        kwarg = {'created_at': timeutils.utcnow(True)}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertTrue(is_up)

    def test_service_is_up_alive_with_updated(self):
        kwarg = {'updated_at': timeutils.utcnow(True)}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertTrue(is_up)

    def test_service_is_up_alive_with_all_three(self):
        kwarg = {'created_at': timeutils.utcnow(True),
                 'updated_at': timeutils.utcnow(True),
                 'last_seen_up': timeutils.utcnow(True)}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertTrue(is_up)

    def test_service_is_up_alive_with_latest_update(self):
        kwarg = {
            'created_at': datetime.datetime(1970, 1, 1,
                                            tzinfo=datetime.timezone.utc),
            'updated_at': datetime.datetime(1970, 1, 1,
                                            tzinfo=datetime.timezone.utc),
            'last_seen_up': timeutils.utcnow(True)
        }
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertTrue(is_up)

    def test_service_is_up_down(self):
        kwarg = {
            'last_seen_up': datetime.datetime(1970, 1, 1,
                                              tzinfo=datetime.timezone.utc)
        }
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertFalse(is_up)

    def test_service_is_up_down_with_create(self):
        kwarg = {'created_at': datetime.datetime(1970, 1, 1,
                                                 tzinfo=datetime.timezone.utc)}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertFalse(is_up)

    def test_service_is_up_down_with_update(self):
        kwarg = {'updated_at': datetime.datetime(1970, 1, 1,
                                                 tzinfo=datetime.timezone.utc)}
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertFalse(is_up)

    def test_service_is_up_down_with_all_three(self):
        kwarg = {
            'last_seen_up': datetime.datetime(1970, 1, 1,
                                              tzinfo=datetime.timezone.utc),
            'created_at': datetime.datetime(1970, 1, 1,
                                            tzinfo=datetime.timezone.utc),
            'updated_at': datetime.datetime(1970, 1, 1,
                                            tzinfo=datetime.timezone.utc)
        }
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertFalse(is_up)

    def test_service_is_up_down_with_old_update(self):
        kwarg = {
            'last_seen_up': datetime.datetime(1970, 1, 1,
                                              tzinfo=datetime.timezone.utc),
            'created_at': timeutils.utcnow(True),
            'updated_at': timeutils.utcnow(True)
        }
        magnum_object = obj_util.get_test_magnum_service_object(
            self.context, **kwarg)
        is_up = self.servicegroup_api.service_is_up(magnum_object)
        self.assertFalse(is_up)



# Copyright 2015 EasyStack, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.


from glanceclient import exc as glance_exception
from novaclient import exceptions as nova_exc
from unittest import mock

from magnum.api import attr_validator
from magnum.common import exception
from magnum.tests import base


class TestAttrValidator(base.BaseTestCase):

    def test_validate_flavor_with_vaild_flavor(self):
        mock_flavor = mock.MagicMock()
        mock_flavor.name = 'test_flavor'
        mock_flavor.id = 'test_flavor_id'
        mock_flavors = [mock_flavor]
        mock_nova = mock.MagicMock()
        mock_nova.flavors.list.return_value = mock_flavors
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        attr_validator.validate_flavor(mock_os_cli, 'test_flavor')
        self.assertTrue(mock_nova.flavors.list.called)

    def test_validate_flavor_with_none_flavor(self):
        mock_flavor = mock.MagicMock()
        mock_flavor.name = 'test_flavor'
        mock_flavor.id = 'test_flavor_id'
        mock_flavors = [mock_flavor]
        mock_nova = mock.MagicMock()
        mock_nova.flavors.list.return_value = mock_flavors
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        attr_validator.validate_flavor(mock_os_cli, None)
        self.assertEqual(False, mock_nova.flavors.list.called)

    def test_validate_flavor_with_invalid_flavor(self):
        mock_flavor = mock.MagicMock()
        mock_flavor.name = 'test_flavor_not_equal'
        mock_flavor.id = 'test_flavor_id_not_equal'
        mock_flavors = [mock_flavor]
        mock_nova = mock.MagicMock()
        mock_nova.flavors.list.return_value = mock_flavors
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        self.assertRaises(exception.FlavorNotFound,
                          attr_validator.validate_flavor,
                          mock_os_cli, 'test_flavor')

    def test_validate_external_network_with_valid_network(self):
        mock_networks = {'networks': [{'name': 'test_ext_net',
                         'id': 'test_ext_net_id'}]}
        mock_neutron = mock.MagicMock()
        mock_neutron.list_networks.return_value = mock_networks
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        attr_validator.validate_external_network(mock_os_cli, 'test_ext_net')
        self.assertTrue(mock_neutron.list_networks.called)

    def test_validate_external_network_with_multiple_valid_network(self):
        mock_networks = {'networks':
                         [{'name': 'test_ext_net', 'id': 'test_ext_net_id1'},
                          {'name': 'test_ext_net', 'id': 'test_ext_net_id2'}]}
        mock_neutron = mock.MagicMock()
        mock_neutron.list_networks.return_value = mock_networks
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.Conflict,
                          attr_validator.validate_external_network,
                          mock_os_cli, 'test_ext_net')

    def test_validate_external_network_with_invalid_network(self):
        mock_networks = {'networks': [{'name': 'test_ext_net_not_equal',
                         'id': 'test_ext_net_id_not_equal'}]}
        mock_neutron = mock.MagicMock()
        mock_neutron.list_networks.return_value = mock_networks
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.ExternalNetworkNotFound,
                          attr_validator.validate_external_network,
                          mock_os_cli, 'test_ext_net')

    def test_validate_fixed_network_with_valid_network(self):
        mock_networks = {'networks': [{'name': 'test_net',
                         'id': 'test_net_id'}]}
        mock_neutron = mock.MagicMock()
        mock_neutron.list_networks.return_value = mock_networks
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertEqual('test_net_id',
                         attr_validator.validate_fixed_network(mock_os_cli,
                                                               'test_net'))
        self.assertTrue(mock_neutron.list_networks.called)

    def test_validate_fixed_network_with_multiple_valid_network(self):
        mock_networks = {
            'networks': [{'name': 'test_net',
                          'id': 'test_net_id1'},
                         {'name': 'test_net',
                          'id': 'test_net_id2'}],
        }
        mock_neutron = mock.MagicMock()
        mock_neutron.list_networks.return_value = mock_networks
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.Conflict,
                          attr_validator.validate_fixed_network,
                          mock_os_cli, 'test_net')

    def test_validate_fixed_network_with_invalid_network(self):
        mock_networks = {'networks': [{'name': 'test_net_not_equal',
                         'id': 'test_net_id_not_equal'}]}
        mock_neutron = mock.MagicMock()
        mock_neutron.list_networks.return_value = mock_networks
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.FixedNetworkNotFound,
                          attr_validator.validate_fixed_network,
                          mock_os_cli, 'test_net')

    def test_validate_fixed_subnet_with_valid_subnet(self):
        mock_neutron = mock.MagicMock()
        mock_subnets = {'subnets': [{'name': 'test_subnet',
                                     'id': 'test_subnet_id',
                                     'network_id': 'test_net_id'}]}
        mock_neutron.list_subnets.return_value = mock_subnets
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertEqual('test_subnet_id',
                         attr_validator.validate_fixed_subnet(mock_os_cli,
                                                              'test_subnet'))
        mock_neutron.list_subnets.assert_called_with()

    def test_validate_fixed_subnet_with_invalid_subnet(self):
        mock_neutron = mock.MagicMock()
        mock_subnets = {'subnets': [{'name': 'test_subnet',
                                     'id': 'test_subnet_id',
                                     'network_id': 'test_net_id'}]}
        mock_neutron.list_subnets.return_value = mock_subnets
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.FixedSubnetNotFound,
                          attr_validator.validate_fixed_subnet,
                          mock_os_cli, 'test_subnet_not_found')

    def test_validate_fixed_subnet_with_multiple_valid_subnet(self):
        mock_neutron = mock.MagicMock()
        mock_subnets = {'subnets': [{'name': 'test_subnet',
                                     'id': 'test_subnet_id',
                                     'network_id': 'test_net_id'},
                                    {'name': 'test_subnet',
                                     'id': 'test_subnet_id2',
                                     'network_id': 'test_net_id2'}]}
        mock_neutron.list_subnets.return_value = mock_subnets
        mock_os_cli = mock.MagicMock()
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.Conflict,
                          attr_validator.validate_fixed_subnet,
                          mock_os_cli, 'test_subnet')

    def test_validate_keypair_with_no_keypair(self):
        mock_keypair = mock.MagicMock()
        mock_keypair.id = None
        mock_nova = mock.MagicMock()
        mock_nova.keypairs.get.return_value = mock_keypair
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        attr_validator.validate_keypair(mock_os_cli, None)

    def test_validate_keypair_with_valid_keypair(self):
        mock_keypair = mock.MagicMock()
        mock_keypair.id = 'test-keypair'
        mock_nova = mock.MagicMock()
        mock_nova.keypairs.get.return_value = mock_keypair
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        attr_validator.validate_keypair(mock_os_cli, 'test-keypair')

    def test_validate_keypair_with_invalid_keypair(self):
        mock_nova = mock.MagicMock()
        mock_nova.keypairs.get.side_effect = nova_exc.NotFound('test-keypair')
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        self.assertRaises(exception.KeyPairNotFound,
                          attr_validator.validate_keypair,
                          mock_os_cli, 'test_keypair')

    def test_validate_labels_main_no_label(self):
        fake_labels = {}
        attr_validator.validate_labels(fake_labels)

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_with_valid_image_by_name(self, mock_os_res):
        mock_image = {'name': 'fedora-21-coreos-5',
                      'id': 'e33f0988-1730-405e-8401-30cbc8535302',
                      'os_distro': 'fedora-coreos'}
        mock_os_res.return_value = mock_image
        mock_os_cli = mock.MagicMock()
        attr_validator.validate_image(mock_os_cli, 'fedora-21-coreos-5')
        self.assertTrue(mock_os_res.called)

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_with_forbidden_image(self, mock_os_res):
        def glance_side_effect(cli, image, name):
            raise glance_exception.HTTPForbidden()

        mock_os_res.side_effect = glance_side_effect
        mock_os_cli = mock.MagicMock()
        self.assertRaises(exception.ImageNotAuthorized,
                          attr_validator.validate_image, mock_os_cli,
                          'fedora-21-coreos-5')

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_with_valid_image_by_id(self, mock_os_res):
        mock_image = {'name': 'fedora-21-coreos-5',
                      'id': 'e33f0988-1730-405e-8401-30cbc8535302',
                      'os_distro': 'fedora-coreos'}
        mock_os_res.return_value = mock_image
        mock_os_cli = mock.MagicMock()
        attr_validator.validate_image(mock_os_cli,
                                      'e33f0988-1730-405e-8401-30cbc8535302')
        self.assertTrue(mock_os_res.called)

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_with_nonexist_image_by_name(self, mock_os_res):
        mock_os_res.side_effect = exception.ResourceNotFound
        mock_os_cli = mock.MagicMock()
        self.assertRaises(exception.ImageNotFound,
                          attr_validator.validate_image,
                          mock_os_cli, 'fedora-21-coreos-5')

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_with_nonexist_image_by_id(self, mock_os_res):
        mock_os_res.side_effect = glance_exception.NotFound
        mock_os_cli = mock.MagicMock()
        self.assertRaises(exception.ImageNotFound,
                          attr_validator.validate_image,
                          mock_os_cli, 'fedora-21-coreos-5')

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_with_multi_images_same_name(self, mock_os_res):
        mock_os_res.side_effect = exception.Conflict
        mock_os_cli = mock.MagicMock()
        self.assertRaises(exception.Conflict,
                          attr_validator.validate_image,
                          mock_os_cli, 'fedora-21-coreos-5')

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_without_os_distro(self, mock_os_res):
        mock_image = {'name': 'fedora-21-coreos-5',
                      'id': 'e33f0988-1730-405e-8401-30cbc8535302'}
        mock_os_res.return_value = mock_image
        mock_os_cli = mock.MagicMock()
        self.assertRaises(exception.OSDistroFieldNotFound,
                          attr_validator.validate_image,
                          mock_os_cli, 'fedora-21-coreos-5')

    @mock.patch('magnum.api.utils.get_openstack_resource')
    def test_validate_image_when_user_forbidden(self, mock_os_res):
        mock_image = {'name': 'fedora-21-coreos-5',
                      'id': 'e33f0988-1730-405e-8401-30cbc8535302',
                      'os_distro': ''}
        mock_os_res.return_value = mock_image
        mock_os_cli = mock.MagicMock()
        self.assertRaises(exception.OSDistroFieldNotFound,
                          attr_validator.validate_image,
                          mock_os_cli, 'fedora-21-coreos-5')

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_validate_os_resources_with_invalid_flavor(self,
                                                       mock_os_cli):
        mock_cluster_template = {'flavor_id': 'test_flavor'}
        mock_flavor = mock.MagicMock()
        mock_flavor.name = 'test_flavor_not_equal'
        mock_flavor.id = 'test_flavor_id_not_equal'
        mock_flavors = [mock_flavor]
        mock_nova = mock.MagicMock()
        mock_nova.flavors.list.return_value = mock_flavors
        mock_os_cli.nova.return_value = mock_nova
        mock_context = mock.MagicMock()
        self.assertRaises(exception.FlavorNotFound,
                          attr_validator.validate_os_resources,
                          mock_context, mock_cluster_template)

    @mock.patch('magnum.common.clients.OpenStackClients')
    @mock.patch('magnum.api.attr_validator.validators')
    def test_validate_os_resources_without_validator(self, mock_validators,
                                                     mock_os_cli):
        mock_cluster_template = {}
        mock_context = mock.MagicMock()
        attr_validator.validate_os_resources(mock_context,
                                             mock_cluster_template)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_validate_os_resources_with_valid_fixed_subnet(self,
                                                           os_clients_klass):
        mock_cluster_template = {'fixed_network': 'test_net',
                                 'fixed_subnet': 'test_subnet'}
        mock_context = mock.MagicMock()
        mock_os_cli = mock.MagicMock()
        os_clients_klass.return_value = mock_os_cli
        mock_neutron = mock.MagicMock()
        mock_networks = {'networks': [{'name': 'test_net',
                                       'id': 'test_net_id'}]}
        mock_neutron.list_networks.return_value = mock_networks
        mock_subnets = {'subnets': [{'name': 'test_subnet',
                                     'id': 'test_subnet_id',
                                     'network_id': 'test_net_id'}]}
        mock_neutron.list_subnets.return_value = mock_subnets
        mock_os_cli.neutron.return_value = mock_neutron
        attr_validator.validate_os_resources(mock_context,
                                             mock_cluster_template)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_validate_os_resources_with_invalid_fixed_subnet(self,
                                                             os_clients_klass):
        mock_cluster_template = {'fixed_network': 'test_net',
                                 'fixed_subnet': 'test_subnet2'}
        mock_context = mock.MagicMock()
        mock_os_cli = mock.MagicMock()
        os_clients_klass.return_value = mock_os_cli
        mock_neutron = mock.MagicMock()
        mock_networks = {'networks': [{'name': 'test_net',
                                       'id': 'test_net_id'}]}
        mock_neutron.list_networks.return_value = mock_networks
        mock_subnets = {'subnets': [{'name': 'test_subnet',
                                     'id': 'test_subnet_id',
                                     'network_id': 'test_net_id'}]}
        mock_neutron.list_subnets.return_value = mock_subnets
        mock_os_cli.neutron.return_value = mock_neutron
        self.assertRaises(exception.FixedSubnetNotFound,
                          attr_validator.validate_os_resources, mock_context,
                          mock_cluster_template)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_validate_os_resources_with_cluster(self, mock_os_cli):
        mock_cluster_template = {}
        mock_cluster = {
            'keypair': 'test-keypair', 'labels': {'lab1': 'val1'},
            'image_id': 'e33f0988-1730-405e-8401-30cbc8535302'
        }
        mock_keypair = mock.MagicMock()
        mock_keypair.id = 'test-keypair'
        mock_image = {'name': 'fedora-21-coreos-5',
                      'id': 'e33f0988-1730-405e-8401-30cbc8535302',
                      'os_distro': 'fedora-coreos'}
        mock_nova = mock.MagicMock()
        mock_nova.keypairs.get.return_value = mock_keypair
        mock_nova.images.get.return_value = mock_image
        mock_os_cli = mock.MagicMock()
        mock_os_cli.nova.return_value = mock_nova
        mock_context = mock.MagicMock()
        attr_validator.validate_os_resources(mock_context,
                                             mock_cluster_template,
                                             mock_cluster)



# Copyright 2014 Red Hat, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

__all__ = [
    'init',
    'cleanup',
    'set_defaults',
    'add_extra_exmods',
    'clear_extra_exmods',
    'get_allowed_exmods',
    'RequestContextSerializer',
    'get_client',
    'get_server',
    'get_notifier',
]

import socket


import oslo_messaging as messaging
from oslo_messaging.rpc import dispatcher
from oslo_serialization import jsonutils
from oslo_utils import importutils

from magnum.common import context as magnum_context
from magnum.common import exception
import magnum.conf

profiler = importutils.try_import("osprofiler.profiler")

CONF = magnum.conf.CONF
TRANSPORT = None
NOTIFIER = None

ALLOWED_EXMODS = [
    exception.__name__,
]
EXTRA_EXMODS = []


def init(conf):
    global TRANSPORT, NOTIFIER
    exmods = get_allowed_exmods()
    TRANSPORT = messaging.get_rpc_transport(conf,
                                            allowed_remote_exmods=exmods)
    serializer = RequestContextSerializer(JsonPayloadSerializer())
    NOTIFIER = messaging.Notifier(TRANSPORT, serializer=serializer)


def cleanup():
    global TRANSPORT, NOTIFIER
    assert TRANSPORT is not None
    assert NOTIFIER is not None
    TRANSPORT.cleanup()
    TRANSPORT = NOTIFIER = None


def set_defaults(control_exchange):
    messaging.set_transport_defaults(control_exchange)


def add_extra_exmods(*args):
    EXTRA_EXMODS.extend(args)


def clear_extra_exmods():
    del EXTRA_EXMODS[:]


def get_allowed_exmods():
    return ALLOWED_EXMODS + EXTRA_EXMODS


class JsonPayloadSerializer(messaging.NoOpSerializer):
    @staticmethod
    def serialize_entity(context, entity):
        return jsonutils.to_primitive(entity, convert_instances=True)


class RequestContextSerializer(messaging.Serializer):

    def __init__(self, base):
        self._base = base

    def serialize_entity(self, context, entity):
        if not self._base:
            return entity
        return self._base.serialize_entity(context, entity)

    def deserialize_entity(self, context, entity):
        if not self._base:
            return entity
        return self._base.deserialize_entity(context, entity)

    def serialize_context(self, context):
        return context.to_dict()

    def deserialize_context(self, context):
        return magnum_context.RequestContext.from_dict(context)


class ProfilerRequestContextSerializer(RequestContextSerializer):
    def serialize_context(self, context):
        _context = super(ProfilerRequestContextSerializer,
                         self).serialize_context(context)

        prof = profiler.get()
        if prof:
            trace_info = {
                "hmac_key": prof.hmac_key,
                "base_id": prof.get_base_id(),
                "parent_id": prof.get_id()
            }
            _context.update({"trace_info": trace_info})

        return _context

    def deserialize_context(self, context):
        trace_info = context.pop("trace_info", None)
        if trace_info:
            profiler.init(**trace_info)

        return super(ProfilerRequestContextSerializer,
                     self).deserialize_context(context)


def get_transport_url(url_str=None):
    return messaging.TransportURL.parse(CONF, url_str)


def get_client(target, version_cap=None, serializer=None, timeout=None):
    assert TRANSPORT is not None
    if profiler:
        serializer = ProfilerRequestContextSerializer(serializer)
    else:
        serializer = RequestContextSerializer(serializer)
    return messaging.get_rpc_client(
        TRANSPORT, target, version_cap=version_cap,
        serializer=serializer, timeout=timeout)


def get_server(target, endpoints, serializer=None):
    assert TRANSPORT is not None
    if profiler:
        serializer = ProfilerRequestContextSerializer(serializer)
    else:
        serializer = RequestContextSerializer(serializer)
    access_policy = dispatcher.DefaultRPCAccessPolicy
    return messaging.get_rpc_server(TRANSPORT,
                                    target,
                                    endpoints,
                                    executor='eventlet',
                                    serializer=serializer,
                                    access_policy=access_policy)


def get_notifier(service='container-infra', host=None, publisher_id=None):
    assert NOTIFIER is not None
    myhost = CONF.host
    if myhost is None:
        myhost = socket.getfqdn()
    if not publisher_id:
        publisher_id = "%s.%s" % (service, host or myhost)
    return NOTIFIER.prepare(publisher_id=publisher_id)



# Copyright 2019 Catalyst Cloud Ltd.
#
#    Licensed under the Apache License, Version 2.0 (the "License");
#    you may not use this file except in compliance with the License.
#    You may obtain a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS,
#    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#    See the License for the specific language governing permissions and
#    limitations under the License.

from unittest import mock

from magnum.common import exception
from magnum.common import neutron
from magnum import objects
from magnum.tests import base
from magnum.tests.unit.db import utils


class NeutronTest(base.TestCase):
    def setUp(self):
        super(NeutronTest, self).setUp()

        cluster_dict = utils.get_test_cluster(node_count=1)
        nodegroups_dict = utils.get_nodegroups_for_cluster(node_count=1)
        self.cluster = objects.Cluster(self.context, **cluster_dict)
        self.nodegroups = [
            objects.NodeGroup(self.context, **nodegroups_dict['master']),
            objects.NodeGroup(self.context, **nodegroups_dict['worker'])
        ]

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_floatingip(self, mock_clients):
        mock_nclient = mock.MagicMock()
        fake_port_id = "b4518944-c2cf-4c69-a1e3-774041fd5d14"
        fake_fip_id = "0f8c6849-af85-424c-aa8e-745ade9a46a7"
        mock_nclient.list_floatingips.return_value = {
            'floatingips': [
                {
                    'router_id': '6ed4f7ef-b8c3-4711-93cf-d53cf0e8bdf5',
                    'status': 'ACTIVE',
                    'description': 'Floating IP for Kubernetes external '
                                   'service ad3080723f1c211e88adbfa163ee1203 '
                                   'from cluster %s' % self.cluster.uuid,
                    'tags': [],
                    'tenant_id': 'cd08a539b7c845ddb92c5d08752101d1',
                    'floating_network_id': 'd0b9a8c5-33e5-4ce1-869a-1e2ec7c2f'
                                           '74b',
                    'port_details': {
                        'status': 'ACTIVE',
                        'name': 'test-k8s-master',
                        'admin_state_up': True,
                        'network_id': '7b9110b5-90a2-40bc-b892-07d641387760 ',
                        'device_owner': 'compute:nova',
                        'mac_address': 'fa:16:3e:6f:ad:6c',
                        'device_id': 'a5c1689f-dd76-4164-8562-6990071701cd'
                    },
                    'fixed_ip_address': '10.0.0.4',
                    'floating_ip_address': '172.24.4.74',
                    'revision_number': 14,
                    'project_id': 'cd08a539b7c845ddb92c5d08752101d1',
                    'port_id': fake_port_id,
                    'id': fake_fip_id
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        neutron.delete_floatingip(self.context, fake_port_id, self.cluster)

        mock_nclient.delete_floatingip.assert_called_once_with(fake_fip_id)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_floatingip_empty(self, mock_clients):
        mock_nclient = mock.MagicMock()
        fake_port_id = "b4518944-c2cf-4c69-a1e3-774041fd5d14"
        mock_nclient.list_floatingips.return_value = {
            'floatingips': []
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        neutron.delete_floatingip(self.context, fake_port_id, self.cluster)

        self.assertFalse(mock_nclient.delete_floatingip.called)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_delete_floatingip_exception(self, mock_clients):
        mock_nclient = mock.MagicMock()
        fake_port_id = "b4518944-c2cf-4c69-a1e3-774041fd5d14"
        fake_fip_id = "0f8c6849-af85-424c-aa8e-745ade9a46a7"
        mock_nclient.list_floatingips.return_value = {
            'floatingips': [
                {
                    'router_id': '6ed4f7ef-b8c3-4711-93cf-d53cf0e8bdf5',
                    'status': 'ACTIVE',
                    'description': 'Floating IP for Kubernetes external '
                                   'service ad3080723f1c211e88adbfa163ee1203 '
                                   'from cluster %s' % self.cluster.uuid,
                    'tags': [],
                    'tenant_id': 'cd08a539b7c845ddb92c5d08752101d1',
                    'floating_network_id': 'd0b9a8c5-33e5-4ce1-869a-1e2ec7c2f'
                                           '74b',
                    'port_details': {
                        'status': 'ACTIVE',
                        'name': 'test-k8s-master',
                        'admin_state_up': True,
                        'network_id': '7b9110b5-90a2-40bc-b892-07d641387760 ',
                        'device_owner': 'compute:nova',
                        'mac_address': 'fa:16:3e:6f:ad:6c',
                        'device_id': 'a5c1689f-dd76-4164-8562-6990071701cd'
                    },
                    'fixed_ip_address': '10.0.0.4',
                    'floating_ip_address': '172.24.4.74',
                    'revision_number': 14,
                    'project_id': 'cd08a539b7c845ddb92c5d08752101d1',
                    'port_id': fake_port_id,
                    'id': fake_fip_id
                }
            ]
        }
        mock_nclient.delete_floatingip.side_effect = Exception

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        self.assertRaises(
            exception.PreDeletionFailed,
            neutron.delete_floatingip,
            self.context,
            fake_port_id,
            self.cluster
        )

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_external_network_id(self, mock_clients):
        fake_name = "fake_network"
        fake_id = "24fe5da0-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_networks.return_value = {
            'networks': [
                {
                    'id': fake_id,
                    'name': fake_name,
                    'router:external': True
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        network_id = neutron.get_external_network_id(self.context, fake_name)

        self.assertEqual(fake_id, network_id)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_external_network_id_notfound(self, mock_clients):
        fake_name = "fake_network"
        fake_id = "24fe5da0-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_networks.return_value = {
            'networks': [
                {
                    'id': fake_id,
                    'name': fake_name,
                    'router:external': True
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        self.assertRaises(
            exception.ExternalNetworkNotFound,
            neutron.get_external_network_id,
            self.context,
            "another_network"
        )

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_external_network_id_conflict(self, mock_clients):
        fake_name = "fake_network"
        fake_id_1 = "24fe5da0-1ac0-11e9-84cd-00224d6b7bc1"
        fake_id_2 = "93781f82-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_networks.return_value = {
            'networks': [
                {
                    'id': fake_id_1,
                    'name': fake_name,
                    'router:external': True
                },
                {
                    'id': fake_id_2,
                    'name': fake_name,
                    'router:external': True
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        self.assertRaises(
            exception.Conflict,
            neutron.get_external_network_id,
            self.context,
            fake_name
        )

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_fixed_network_name(self, mock_clients):
        fake_name = "fake_network"
        fake_id = "24fe5da0-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_networks.return_value = {
            'networks': [
                {
                    'id': fake_id,
                    'name': fake_name,
                    'router:external': False
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        network_name = neutron.get_fixed_network_name(self.context, fake_id)

        self.assertEqual(fake_name, network_name)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_fixed_network_name_notfound(self, mock_clients):
        fake_name = "fake_network"
        fake_id = "24fe5da0-1ac0-11e9-84cd-00224d6b7bc1"
        another_fake_id = "34fe5da0-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_networks.return_value = {
            'networks': [
                {
                    'id': fake_id,
                    'name': fake_name,
                    'router:external': False
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        self.assertRaises(
            exception.FixedNetworkNotFound,
            neutron.get_fixed_network_name,
            self.context,
            another_fake_id
        )

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_fixed_subnet_id(self, mock_clients):
        fake_name = "fake_subnet"
        fake_id = "35ee5da0-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_subnets.return_value = {
            'subnets': [
                {
                    'id': fake_id,
                    'name': fake_name,
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        subnet_id = neutron.get_fixed_subnet_id(self.context, fake_name)

        self.assertEqual(fake_id, subnet_id)

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_fixed_subnet_id_notfound(self, mock_clients):
        fake_name = "fake_subnet"
        fake_id = "35ee5da0-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_subnets.return_value = {
            'subnets': [
                {
                    'id': fake_id,
                    'name': fake_name,
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        self.assertRaises(
            exception.FixedSubnetNotFound,
            neutron.get_fixed_subnet_id,
            self.context,
            "another_subnet"
        )

    @mock.patch('magnum.common.clients.OpenStackClients')
    def test_get_fixed_subnet_id_conflict(self, mock_clients):
        fake_name = "fake_subnet"
        fake_id_1 = "35ee5da0-1ac0-11e9-84cd-00224d6b7bc1"
        fake_id_2 = "93781f82-1ac0-11e9-84cd-00224d6b7bc1"
        mock_nclient = mock.MagicMock()
        mock_nclient.list_subnets.return_value = {
            'subnets': [
                {
                    'id': fake_id_1,
                    'name': fake_name,
                },
                {
                    'id': fake_id_2,
                    'name': fake_name,
                }
            ]
        }

        osc = mock.MagicMock()
        mock_clients.return_value = osc
        osc.neutron.return_value = mock_nclient

        self.assertRaises(
            exception.Conflict,
            neutron.get_fixed_subnet_id,
            self.context,
            fake_name
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from oslo_config import cfg

drivers_group = cfg.OptGroup(name='drivers',
                             title='Options for the Drivers')

drivers_opts = [
    cfg.BoolOpt('verify_ca',
                default=True,
                help='Indicates whether the cluster nodes validate the '
                     'Certificate Authority when making requests to the '
                     'OpenStack APIs (Keystone, Magnum, Heat). If you have '
                     'self-signed certificates for the OpenStack APIs or '
                     'you have your own Certificate Authority and you '
                     'have not installed the Certificate Authority to all '
                     'nodes, you may need to disable CA validation by '
                     'setting this flag to False.'),
    cfg.StrOpt('openstack_ca_file',
               default="",
               help='Path to the OpenStack CA-bundle file to pass and '
                    'install in all cluster nodes.'),
    cfg.ListOpt('disabled_drivers',
                default=[],
                help='Disabled driver entry points. If empty, then all '
                     'available drivers are enabled.'
                ),
    cfg.ListOpt('enabled_beta_drivers',
                default=[],
                help='List of beta drivers to enable. Beta drivers are not '
                     'intended for production.'
                ),
]


def register_opts(conf):
    conf.register_group(drivers_group)
    conf.register_opts(drivers_opts, group=drivers_group)


def list_opts():
    return {
        drivers_group: drivers_opts,
    }



# Licensed under the Apache License, Version 2.0 (the "License"); you may not
# use this file except in compliance with the License. You may obtain a copy
# of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import itertools
from oslo_config import cfg

DEFAULT_CERT_MANAGER = 'barbican'
TLS_STORAGE_DEFAULT = '/var/lib/magnum/certificates/'

certificates_group = cfg.OptGroup(name='certificates',
                                  title='Certificate options for the '
                                        'cert manager.')

cert_manager_opts = [
    cfg.StrOpt('cert_manager_type',
               default=DEFAULT_CERT_MANAGER,
               help='Certificate Manager plugin.')
]

local_cert_manager_opts = [
    cfg.StrOpt('storage_path',
               default=TLS_STORAGE_DEFAULT,
               help='Absolute path of the certificate storage directory.')
]

ALL_OPTS = list(itertools.chain(
    cert_manager_opts,
    local_cert_manager_opts
))


def register_opts(conf):
    conf.register_group(certificates_group)
    conf.register_opts(ALL_OPTS, group=certificates_group)


def list_opts():
    return {
        certificates_group: ALL_OPTS
    }



#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import pecan
from wsme import types as wtypes

from magnum.api.controllers import base
from magnum.api import expose
from magnum.common import exception
from magnum.common import policy
from magnum.i18n import _
from magnum import objects


class Stats(base.APIBase):

    clusters = wtypes.IntegerType(minimum=0)
    nodes = wtypes.IntegerType(minimum=0)

    def __init__(self, **kwargs):
        self.fields = []
        for field in objects.Stats.fields:
            # Skip fields we do not expose.
            if not hasattr(self, field):
                continue
            self.fields.append(field)
            setattr(self, field, kwargs.get(field, wtypes.Unset))

    @classmethod
    def convert(cls, rpc_stats):
        return Stats(**rpc_stats.as_dict())


class StatsController(base.Controller):
    """REST controller for Stats."""
    def __init__(self, **kwargs):
        super(StatsController, self).__init__()

    @expose.expose(Stats, wtypes.text, wtypes.text)
    def get_all(self, project_id=None, type="cluster"):
        """Retrieve magnum stats.

        """
        context = pecan.request.context
        policy.enforce(context, 'stats:get_all', action='stats:get_all')
        allowed_stats = ["cluster"]

        if type.lower() not in allowed_stats:
            msg = _("Invalid stats type. Allowed values are '%s'")
            allowed_str = ','.join(allowed_stats)
            raise exception.InvalidParameterValue(err=msg % allowed_str)

        # 1.If the requester is not an admin and trying to request stats for
        # different tenant, then reject the request
        # 2.If the requester is not an admin and project_id was not provided,
        # then return self stats
        if not context.is_admin:
            project_id = project_id if project_id else context.project_id
            if project_id != context.project_id:
                raise exception.NotAuthorized()

        stats = objects.Stats.get_cluster_stats(context, project_id)
        return Stats.convert(stats)



# Copyright 2015 NEC Corporation.  All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from unittest import mock

from magnum.common import exception
from magnum.conductor.handlers.common import cert_manager
from magnum.tests import base
from oslo_config import cfg

import magnum.conf
import os
import stat
import tempfile


CONF = magnum.conf.CONF


class CertManagerTestCase(base.BaseTestCase):
    def setUp(self):
        super(CertManagerTestCase, self).setUp()

        cert_manager_patcher = mock.patch.object(cert_manager, 'cert_manager')
        self.cert_manager = cert_manager_patcher.start()
        self.addCleanup(cert_manager_patcher.stop)

        self.cert_manager_backend = mock.MagicMock()
        self.cert_manager.get_backend.return_value = self.cert_manager_backend

        self.cert_manager_backend.CertManager = mock.MagicMock()
        self.CertManager = self.cert_manager_backend.CertManager

    @mock.patch('magnum.common.x509.operations.generate_ca_certificate')
    @mock.patch('magnum.common.short_id.generate_id')
    def test_generate_ca_cert(self, mock_generate_id, mock_generate_ca_cert):
        expected_ca_name = 'ca-name'
        expected_ca_password = 'password'
        expected_ca_cert = {
            'private_key': 'private_key', 'certificate': 'certificate'}
        expected_ca_cert_ref = 'ca_cert_ref'

        mock_generate_id.return_value = expected_ca_password
        mock_generate_ca_cert.return_value = expected_ca_cert

        self.CertManager.store_cert.return_value = expected_ca_cert_ref
        self.assertEqual((expected_ca_cert_ref, expected_ca_cert,
                          expected_ca_password),
                         cert_manager._generate_ca_cert(expected_ca_name))

        mock_generate_ca_cert.assert_called_once_with(
            expected_ca_name, encryption_password=expected_ca_password)
        self.CertManager.store_cert.assert_called_once_with(
            certificate=expected_ca_cert['certificate'],
            private_key=expected_ca_cert['private_key'],
            private_key_passphrase=expected_ca_password,
            name=expected_ca_name,
            context=None
        )

    @mock.patch('magnum.common.x509.operations.generate_client_certificate')
    @mock.patch('magnum.common.short_id.generate_id')
    def test_generate_client_cert(self, mock_generate_id, mock_generate_cert):
        expected_name = 'admin'
        expected_organization_name = 'system:masters'
        expected_ca_name = 'ca-name'
        expected_password = 'password'
        expected_ca_password = 'ca-password'
        expected_cert = {
            'private_key': 'private_key', 'certificate': 'certificate'}
        expected_ca_cert = {
            'private_key': 'ca_private_key', 'certificate': 'ca_certificate'}
        expected_cert_ref = 'cert_ref'

        mock_generate_id.return_value = expected_password
        mock_generate_cert.return_value = expected_cert

        self.CertManager.store_cert.return_value = expected_cert_ref

        self.assertEqual(
            expected_cert_ref,
            cert_manager._generate_client_cert(
                expected_ca_name,
                expected_ca_cert,
                expected_ca_password))

        mock_generate_cert.assert_called_once_with(
            expected_ca_name,
            expected_name,
            expected_organization_name,
            expected_ca_cert['private_key'],
            encryption_password=expected_password,
            ca_key_password=expected_ca_password,
        )
        self.CertManager.store_cert.assert_called_once_with(
            certificate=expected_cert['certificate'],
            private_key=expected_cert['private_key'],
            private_key_passphrase=expected_password,
            name=cert_manager.CONDUCTOR_CLIENT_NAME,
            context=None
        )

    def _test_generate_certificates(self,
                                    expected_ca_name,
                                    mock_cluster,
                                    mock_generate_ca_cert,
                                    mock_generate_client_cert):
        expected_ca_password = 'ca-password'
        expected_ca_cert = {
            'private_key': 'ca_private_key', 'certificate': 'ca_certificate'}
        expected_cert_ref = 'cert_ref'
        expected_ca_cert_ref = 'ca-cert-ref'

        mock_generate_ca_cert.return_value = (expected_ca_cert_ref,
                                              expected_ca_cert,
                                              expected_ca_password)
        mock_generate_client_cert.return_value = expected_cert_ref

        cert_manager.generate_certificates_to_cluster(mock_cluster)
        self.assertEqual(expected_ca_cert_ref, mock_cluster.ca_cert_ref)
        self.assertEqual(expected_cert_ref, mock_cluster.magnum_cert_ref)

        mock_generate_ca_cert.assert_called_with(expected_ca_name,
                                                 context=None)
        mock_generate_client_cert.assert_called_once_with(
            expected_ca_name, expected_ca_cert, expected_ca_password,
            context=None)

    @mock.patch('magnum.conductor.handlers.common.cert_manager.'
                '_generate_client_cert')
    @mock.patch('magnum.conductor.handlers.common.cert_manager.'
                '_generate_ca_cert')
    def test_generate_certificates(self, mock_generate_ca_cert,
                                   mock_generate_client_cert):
        expected_ca_name = 'ca-name'
        mock_cluster = mock.MagicMock()
        mock_cluster.name = expected_ca_name

        self._test_generate_certificates(expected_ca_name,
                                         mock_cluster,
                                         mock_generate_ca_cert,
                                         mock_generate_client_cert)

    @mock.patch('magnum.conductor.handlers.common.cert_manager.'
                '_generate_client_cert')
    @mock.patch('magnum.conductor.handlers.common.cert_manager.'
                '_generate_ca_cert')
    def test_generate_certificates_without_name(self, mock_generate_ca_cert,
                                                mock_generate_client_cert):
        expected_ca_name = 'ca-uuid'
        mock_cluster = mock.MagicMock()
        mock_cluster.name = None
        mock_cluster.uuid = expected_ca_name

        self._test_generate_certificates(expected_ca_name,
                                         mock_cluster,
                                         mock_generate_ca_cert,
                                         mock_generate_client_cert)

    @mock.patch('magnum.conductor.handlers.common.cert_manager.'
                '_get_issuer_name')
    def test_generate_certificates_with_error(self, mock_get_issuer_name):
        mock_cluster = mock.MagicMock()
        mock_get_issuer_name.side_effect = exception.MagnumException()

        self.assertRaises(exception.CertificatesToClusterFailed,
                          cert_manager.generate_certificates_to_cluster,
                          mock_cluster)

    @mock.patch('magnum.common.x509.operations.sign')
    def test_sign_node_certificate(self, mock_x509_sign):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_ca_cert = mock.MagicMock()
        mock_ca_cert.get_private_key.return_value = mock.sentinel.priv_key
        passphrase = mock.sentinel.passphrase
        mock_ca_cert.get_private_key_passphrase.return_value = passphrase
        self.CertManager.get_cert.return_value = mock_ca_cert
        mock_csr = mock.MagicMock()
        mock_x509_sign.return_value = mock.sentinel.signed_cert
        cluster_ca_cert = cert_manager.sign_node_certificate(mock_cluster,
                                                             mock_csr)

        self.CertManager.get_cert.assert_called_once_with(
            mock_cluster.ca_cert_ref, resource_ref=mock_cluster.uuid,
            context=None)
        mock_x509_sign.assert_called_once_with(mock_csr, mock_cluster.name,
                                               mock.sentinel.priv_key,
                                               passphrase)
        self.assertEqual(mock.sentinel.signed_cert, cluster_ca_cert)

    @mock.patch('magnum.common.x509.operations.sign')
    def test_sign_node_certificate_without_cluster_name(self, mock_x509_sign):
        mock_cluster = mock.MagicMock()
        mock_cluster.name = None
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_ca_cert = mock.MagicMock()
        mock_ca_cert.get_private_key.return_value = mock.sentinel.priv_key
        passphrase = mock.sentinel.passphrase
        mock_ca_cert.get_private_key_passphrase.return_value = passphrase
        self.CertManager.get_cert.return_value = mock_ca_cert
        mock_csr = mock.MagicMock()
        mock_x509_sign.return_value = mock.sentinel.signed_cert

        cluster_ca_cert = cert_manager.sign_node_certificate(mock_cluster,
                                                             mock_csr)

        self.CertManager.get_cert.assert_called_once_with(
            mock_cluster.ca_cert_ref, resource_ref=mock_cluster.uuid,
            context=None)
        mock_x509_sign.assert_called_once_with(mock_csr, mock_cluster.uuid,
                                               mock.sentinel.priv_key,
                                               passphrase)
        self.assertEqual(mock.sentinel.signed_cert, cluster_ca_cert)

    def test_get_cluster_ca_certificate(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_ca_cert = mock.MagicMock()
        self.CertManager.get_cert.return_value = mock_ca_cert

        cluster_ca_cert = cert_manager.get_cluster_ca_certificate(mock_cluster)

        self.CertManager.get_cert.assert_called_once_with(
            mock_cluster.ca_cert_ref, resource_ref=mock_cluster.uuid,
            context=None)
        self.assertEqual(mock_ca_cert, cluster_ca_cert)

    def test_get_cluster_ca_certificate_ca_cert_type(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_ca_cert = mock.MagicMock()
        self.CertManager.get_cert.return_value = mock_ca_cert

        cluster_ca_cert = cert_manager.get_cluster_ca_certificate(
            mock_cluster, ca_cert_type="front-proxy")

        self.CertManager.get_cert.assert_called_once_with(
            mock_cluster.front_proxy_ca_cert_ref,
            resource_ref=mock_cluster.uuid, context=None)
        self.assertEqual(mock_ca_cert, cluster_ca_cert)

    def test_get_cluster_magnum_cert(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_magnum_cert = mock.MagicMock()
        self.CertManager.get_cert.return_value = mock_magnum_cert

        cluster_magnum_cert = cert_manager.get_cluster_magnum_cert(
            mock_cluster)

        self.CertManager.get_cert.assert_called_once_with(
            mock_cluster.magnum_cert_ref, resource_ref=mock_cluster.uuid,
            context=None)
        self.assertEqual(mock_magnum_cert, cluster_magnum_cert)

    def test_create_client_files_notin_cache(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_dir = tempfile.mkdtemp()
        cert_dir = os.path.join(mock_dir,
                                mock_cluster.uuid)
        cfg.CONF.set_override("temp_cache_dir", mock_dir, group='cluster')

        mock_ca_return = '%s/ca.crt' % cert_dir
        mock_key_return = '%s/client.key' % cert_dir
        mock_magnum_return = '%s/client.crt' % cert_dir

        mock_cert = mock.MagicMock()
        mock_cert.get_certificate.return_value = "some_content"
        mock_cert.get_decrypted_private_key.return_value = "some_key"
        self.CertManager.get_cert.return_value = \
            mock_cert

        # Test that directory and files DNE
        self.assertEqual(False, os.path.isdir(cert_dir))
        self.assertEqual(False, os.path.isfile(mock_ca_return))
        self.assertEqual(False, os.path.isfile(mock_key_return))
        self.assertEqual(False, os.path.isfile(mock_magnum_return))

        (cluster_ca_cert, cluster_key, cluster_magnum_cert) = \
            cert_manager.create_client_files(mock_cluster)

        # Test the directory and files were created
        self.assertEqual(True, os.path.isdir(cert_dir))
        self.assertEqual(True, os.path.isfile(mock_ca_return))
        self.assertEqual(True, os.path.isfile(mock_key_return))
        self.assertEqual(True, os.path.isfile(mock_magnum_return))

        # Test that all functions were called in the if not conditional
        self.assertEqual(self.CertManager.get_cert.call_count, 2)
        self.assertEqual(mock_cert.get_certificate.call_count, 2)
        self.assertEqual(mock_cert.get_decrypted_private_key.call_count, 1)

        # Test that contents were written to files & returned properly
        cluster_ca_cert.seek(0)
        cluster_key.seek(0)
        cluster_magnum_cert.seek(0)
        self.assertEqual(mock_cert.get_certificate.return_value,
                         cluster_ca_cert.read())
        self.assertEqual(mock_cert.get_decrypted_private_key.return_value,
                         cluster_key.read())
        self.assertEqual(mock_cert.get_certificate.return_value,
                         cluster_magnum_cert.read())

    @mock.patch('magnum.conductor.handlers.common.cert_manager.LOG')
    def test_create_client_files_temp_no_dir(self, mock_logging):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"

        cfg.CONF.set_override("temp_cache_dir", "", group='cluster')

        mock_cert = mock.MagicMock()
        mock_cert.get_certificate.return_value = "some_content"
        mock_cert.get_decrypted_private_key.return_value = "some_key"
        self.CertManager.get_cert.return_value = \
            mock_cert

        (cluster_ca_cert, cluster_key, cluster_magnum_cert) = \
            cert_manager.create_client_files(mock_cluster)

        mock_logging.debug.assert_called_once_with(
            "Certificates will not be cached in the filesystem: "
            "they will be created as tempfiles.")
        self.assertEqual(self.CertManager.get_cert.call_count, 2)
        self.assertEqual(mock_cert.get_certificate.call_count, 2)
        self.assertEqual(mock_cert.get_decrypted_private_key.call_count, 1)

        # Test that contents were written to files & returned properly
        cluster_ca_cert.seek(0)
        cluster_key.seek(0)
        cluster_magnum_cert.seek(0)
        self.assertEqual(mock_cert.get_certificate.return_value,
                         cluster_ca_cert.read())
        self.assertEqual(mock_cert.get_decrypted_private_key.return_value,
                         cluster_key.read())
        self.assertEqual(mock_cert.get_certificate.return_value,
                         cluster_magnum_cert.read())

        # Test for certs and keys that might be returned in binary
        mock_cert.get_certificate.return_value = b"byte_content"
        mock_cert.get_decrypted_private_key.return_value = b"byte_key"
        ca_cert_text = magnum_cert_text = \
            mock_cert.get_certificate.return_value.decode('UTF-8')
        magnum_key_text = \
            mock_cert.get_decrypted_private_key.return_value.decode('UTF-8')
        (cluster_ca_cert, cluster_key, cluster_magnum_cert) = \
            cert_manager.create_client_files(mock_cluster)
        cluster_ca_cert.seek(0)
        cluster_key.seek(0)
        cluster_magnum_cert.seek(0)
        self.assertEqual(ca_cert_text, cluster_ca_cert.read())
        self.assertEqual(magnum_key_text, cluster_key.read())
        self.assertEqual(magnum_cert_text, cluster_magnum_cert.read())

    def test_create_client_files_in_cache(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_dir = tempfile.mkdtemp()
        cfg.CONF.set_override("temp_cache_dir", mock_dir, group='cluster')

        mock_cert = mock.MagicMock()
        mock_cert.get_certificate.return_value = "some_content"
        mock_cert.get_decrypted_private_key.return_value = "some_key"
        self.CertManager.get_cert.return_value = \
            mock_cert

        # First call creates directory and writes files
        (cluster_ca_cert, cluster_key, cluster_magnum_cert) = \
            cert_manager.create_client_files(mock_cluster)

        # Establish call count baseline
        self.assertEqual(self.CertManager.get_cert.call_count, 2)
        self.assertEqual(mock_cert.get_certificate.call_count, 2)
        self.assertEqual(mock_cert.get_decrypted_private_key.call_count, 1)

        # Second call to create_client_files for same cluster should enter else
        # conditional, open cached file and return file contents unchanged.
        (cluster_ca_cert, cluster_key, cluster_magnum_cert) = \
            cert_manager.create_client_files(mock_cluster)

        # Test that function call count did not increase.
        self.assertEqual(self.CertManager.get_cert.call_count, 2)
        self.assertEqual(mock_cert.get_certificate.call_count, 2)
        self.assertEqual(mock_cert.get_decrypted_private_key.call_count, 1)

        # Check that original file contents/return values have not changed
        self.assertEqual(mock_cert.get_certificate.return_value,
                         cluster_ca_cert.read())
        self.assertEqual(mock_cert.get_decrypted_private_key.return_value,
                         cluster_key.read())
        self.assertEqual(mock_cert.get_certificate.return_value,
                         cluster_magnum_cert.read())

    def test_create_client_files_set_file_permissions(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_dir = tempfile.mkdtemp()
        cert_dir = os.path.join(mock_dir,
                                mock_cluster.uuid)
        cfg.CONF.set_override("temp_cache_dir", mock_dir, group='cluster')

        mock_ca_return = '%s/ca.crt' % cert_dir
        mock_key_return = '%s/client.key' % cert_dir
        mock_magnum_return = '%s/client.crt' % cert_dir

        mock_cert = mock.MagicMock()
        mock_cert.get_certificate.return_value = "some_content"
        mock_cert.get_decrypted_private_key.return_value = "some_key"
        self.CertManager.get_cert.return_value = \
            mock_cert

        cert_manager.create_client_files(mock_cluster)

        ca_permission = stat.S_IMODE(os.lstat(mock_ca_return).st_mode)
        self.assertEqual(ca_permission, 0o600)
        key_permission = stat.S_IMODE(os.lstat(mock_key_return).st_mode)
        self.assertEqual(key_permission, 0o600)
        magnum_permission = stat.S_IMODE(os.lstat(mock_magnum_return).st_mode)
        self.assertEqual(magnum_permission, 0o600)

    def test_delete_certificates(self):
        mock_delete_cert = self.CertManager.delete_cert
        expected_cert_ref = 'cert_ref'
        expected_ca_cert_ref = 'ca_cert_ref'
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_cluster.ca_cert_ref = expected_ca_cert_ref
        mock_cluster.magnum_cert_ref = expected_cert_ref

        cert_manager.delete_certificates_from_cluster(mock_cluster)
        mock_delete_cert.assert_any_call(expected_ca_cert_ref,
                                         resource_ref=mock_cluster.uuid,
                                         context=None)
        mock_delete_cert.assert_any_call(expected_cert_ref,
                                         resource_ref=mock_cluster.uuid,
                                         context=None)

    def test_delete_certificates_if_raise_error(self):
        mock_delete_cert = self.CertManager.delete_cert
        expected_cert_ref = 'cert_ref'
        expected_ca_cert_ref = 'ca_cert_ref'
        mock_cluster = mock.MagicMock()
        mock_cluster.ca_cert_ref = expected_ca_cert_ref
        mock_cluster.magnum_cert_ref = expected_cert_ref

        mock_delete_cert.side_effect = ValueError

        cert_manager.delete_certificates_from_cluster(mock_cluster)
        mock_delete_cert.assert_any_call(expected_ca_cert_ref,
                                         resource_ref=mock_cluster.uuid,
                                         context=None)
        mock_delete_cert.assert_any_call(expected_cert_ref,
                                         resource_ref=mock_cluster.uuid,
                                         context=None)

    def test_delete_certificates_without_cert_ref(self):
        mock_delete_cert = self.CertManager.delete_cert
        mock_cluster = mock.MagicMock()
        mock_cluster.ca_cert_ref = None
        mock_cluster.magnum_cert_ref = None

        cert_manager.delete_certificates_from_cluster(mock_cluster)
        self.assertFalse(mock_delete_cert.called)

    def test_delete_client_files(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_dir = tempfile.mkdtemp()
        cert_dir = os.path.join(mock_dir,
                                mock_cluster.uuid)
        cfg.CONF.set_override("temp_cache_dir", mock_dir, group='cluster')

        mock_ca_return = '%s/ca.crt' % cert_dir
        mock_key_return = '%s/client.key' % cert_dir
        mock_magnum_return = '%s/client.crt' % cert_dir

        mock_cert = mock.MagicMock()
        mock_cert.get_certificate.return_value = "some_content"
        mock_cert.get_decrypted_private_key.return_value = "some_key"
        self.CertManager.get_cert.return_value = \
            mock_cert

        (cluster_ca_cert, cluster_key, cluster_magnum_cert) = \
            cert_manager.create_client_files(mock_cluster)

        # Test the directory and files were created
        self.assertEqual(True, os.path.isdir(cert_dir))
        self.assertEqual(True, os.path.isfile(mock_ca_return))
        self.assertEqual(True, os.path.isfile(mock_key_return))
        self.assertEqual(True, os.path.isfile(mock_magnum_return))

        cert_manager.delete_client_files(mock_cluster)

        # Test that directory and files DNE
        self.assertEqual(False, os.path.isdir(cert_dir))
        self.assertEqual(False, os.path.isfile(mock_ca_return))
        self.assertEqual(False, os.path.isfile(mock_key_return))
        self.assertEqual(False, os.path.isfile(mock_magnum_return))

    def test_delete_client_files_none(self):
        mock_cluster = mock.MagicMock()
        mock_cluster.uuid = "mock_cluster_uuid"
        mock_dir = tempfile.mkdtemp()
        cfg.CONF.set_override("temp_cache_dir", mock_dir, group='cluster')
        cert_dir = os.path.join(mock_dir,
                                mock_cluster.uuid)

        self.assertEqual(True, os.path.isdir(mock_dir))
        self.assertEqual(False, os.path.isdir(cert_dir))

        cert_manager.delete_client_files(mock_cluster)

        self.assertEqual(True, os.path.isdir(mock_dir))
        self.assertEqual(False, os.path.isdir(cert_dir))
